@@ -48,16 +48,11 @@
 # include <sys/event.h>
 #endif
 
-#if defined(HAVE_SYS_EVENTFD_H) && defined(HAVE_EVENTFD)
-# define USE_EVENTFD
-# include <sys/eventfd.h>
-#endif
-
 #ifdef HAVE_REGEX_H
 # include <regex.h>
 #endif
 
-#include <event.h>
+#include <event2/event.h>
 
 #include "logger.h"
 #include "db.h"
@@ -77,6 +72,22 @@
 #endif
 
 
+struct filescanner_command;
+
+typedef int (*cmd_func)(struct filescanner_command *cmd);
+
+struct filescanner_command
+{
+  pthread_mutex_t lck;
+  pthread_cond_t cond;
+
+  cmd_func func;
+
+  int nonblock;
+
+  int ret;
+};
+
 #define F_SCAN_BULK    (1 << 0)
 #define F_SCAN_RESCAN  (1 << 1)
 #define F_SCAN_FAST    (1 << 2)
@@ -87,6 +98,7 @@ enum file_type {
   FILE_IGNORE,
   FILE_REGULAR,
   FILE_PLAYLIST,
+  FILE_SMARTPL,
   FILE_ITUNES,
   FILE_ARTWORK,
   FILE_CTRL_REMOTE,
@@ -108,16 +120,14 @@ struct stacked_dir {
 };
 
 
-#ifdef USE_EVENTFD
-static int exit_efd;
-#else
+static int cmd_pipe[2];
 static int exit_pipe[2];
-#endif
 static int scan_exit;
 static int inofd;
 static struct event_base *evbase_scan;
-static struct event inoev;
-static struct event exitev;
+static struct event *inoev;
+static struct event *exitev;
+static struct event *cmdev;
 static pthread_t tid_scan;
 static struct deferred_pl *playlists;
 static struct stacked_dir *dirstack;
@@ -125,6 +135,19 @@ static struct stacked_dir *dirstack;
 /* Count of files scanned during a bulk scan */
 static int counter;
 
+/* Flag for scan in progress */
+static int scanning;
+
+/* When copying into the lib (eg. if a file is moved to the lib by copying into
+ * a Samba network share) inotify might give us IN_CREATE -> n x IN_ATTRIB ->
+ * IN_CLOSE_WRITE, but we don't want to do any scanning before the
+ * IN_CLOSE_WRITE. So we register new files (by path hashes) in this ring buffer
+ * when we get the IN_CREATE and then ignore the IN_ATTRIB for these files.
+ */
+#define INCOMINGFILES_BUFFER_SIZE 50
+static int incomingfiles_idx;
+static uint32_t incomingfiles_buffer[INCOMINGFILES_BUFFER_SIZE];
+
 /* Forward */
 static void
 bulk_scan(int flags);
@@ -132,6 +155,46 @@ static int
 inofd_event_set(void);
 static void
 inofd_event_unset(void);
+static int
+filescanner_initscan(struct filescanner_command *cmd);
+static int
+filescanner_fullrescan(struct filescanner_command *cmd);
+
+
+/* ---------------------------- COMMAND EXECUTION -------------------------- */
+
+static int
+send_command(struct filescanner_command *cmd)
+{
+  int ret;
+
+  if (!cmd->func)
+    {
+      DPRINTF(E_LOG, L_SCAN, "BUG: cmd->func is NULL!\n");
+      return -1;
+    }
+
+  ret = write(cmd_pipe[1], &cmd, sizeof(cmd));
+  if (ret != sizeof(cmd))
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not send command: %s\n", strerror(errno));
+      return -1;
+    }
+
+  return 0;
+}
+
+static int
+nonblock_command(struct filescanner_command *cmd)
+{
+  int ret;
+
+  ret = send_command(cmd);
+  if (ret < 0)
+    return -1;
+
+  return 0;
+}
 
 static int
 push_dir(struct stacked_dir **s, char *path)
@@ -256,6 +319,9 @@ file_type_get(const char *path) {
   if ((strcasecmp(ext, ".m3u") == 0) || (strcasecmp(ext, ".pls") == 0))
     return FILE_PLAYLIST;
 
+  if (strcasecmp(ext, ".smartpl") == 0)
+    return FILE_SMARTPL;
+
   if (artwork_file_is_artwork(filename))
     return FILE_ARTWORK;
 
@@ -472,7 +538,7 @@ fixup_tags(struct media_file_info *mfi)
   /* Handle TV shows, try to present prettier metadata */
   if (mfi->tv_series_name && strlen(mfi->tv_series_name) != 0)
     {
-      mfi->media_kind = 64;  /* tv show */
+      mfi->media_kind = MEDIA_KIND_TVSHOW;  /* tv show */
 
       /* Default to artist = series_name */
       if (mfi->artist && strlen(mfi->artist) == 0)
@@ -513,7 +579,7 @@ fixup_tags(struct media_file_info *mfi)
       /* fname is left untouched by unicode_fixup_mfi() for
        * obvious reasons, so ensure it is proper UTF-8
        */
-      mfi->title = unicode_fixup_string(mfi->fname);
+      mfi->title = unicode_fixup_string(mfi->fname, "ascii");
       if (mfi->title == mfi->fname)
 	mfi->title = strdup(mfi->fname);
     }
@@ -543,7 +609,7 @@ fixup_tags(struct media_file_info *mfi)
 	  mfi->album_artist_sort = strdup("");
 	}
     }
-  else if (mfi->media_kind == 4) /* Podcast */
+  else if (mfi->media_kind == MEDIA_KIND_PODCAST) /* Podcast */
     {
       if (mfi->album_artist)
 	free(mfi->album_artist);
@@ -573,6 +639,7 @@ filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct
   char *filename;
   time_t stamp;
   int id;
+  char virtual_path[PATH_MAX];
   int ret;
 
   filename = strrchr(path, '/');
@@ -623,28 +690,39 @@ filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct
   mfi->time_modified = mtime;
   mfi->file_size = size;
 
+  if (type & F_SCAN_TYPE_COMPILATION)
+    mfi->compilation = 1;
+  if (type & F_SCAN_TYPE_PODCAST)
+    mfi->media_kind = MEDIA_KIND_PODCAST; /* podcast */
+  if (type & F_SCAN_TYPE_AUDIOBOOK)
+    mfi->media_kind = MEDIA_KIND_AUDIOBOOK; /* audiobook */
+
   if (type & F_SCAN_TYPE_FILE)
     {
-      mfi->data_kind = 0; /* real file */
+      mfi->data_kind = DATA_KIND_FILE;
       ret = scan_metadata_ffmpeg(path, mfi);
     }
   else if (type & F_SCAN_TYPE_URL)
     {
-      mfi->data_kind = 1; /* url/stream */
-#if LIBAVFORMAT_VERSION_MAJOR >= 56 || (LIBAVFORMAT_VERSION_MAJOR == 55 && LIBAVFORMAT_VERSION_MINOR >= 13)
+      mfi->data_kind = DATA_KIND_HTTP;
       ret = scan_metadata_ffmpeg(path, mfi);
-#else
-      ret = scan_metadata_icy(path, mfi);
-#endif
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_SCAN, "Playlist URL is unavailable for probe/metadata, assuming MP3 encoding\n");
+	  mfi->type = strdup("mp3");
+	  mfi->codectype = strdup("mpeg");
+	  mfi->description = strdup("MPEG audio file");
+	  ret = 1;
+	}
     }
   else if (type & F_SCAN_TYPE_SPOTIFY)
     {
-      mfi->data_kind = 2; /* iTunes has no spotify data kind, but we use 2 */
+      mfi->data_kind = DATA_KIND_SPOTIFY;
       ret = mfi->artist && mfi->album && mfi->title;
     }
   else if (type & F_SCAN_TYPE_PIPE)
     {
-      mfi->data_kind = 3; /* iTunes has no pipe data kind, but we use 3 */
+      mfi->data_kind = DATA_KIND_PIPE;
       mfi->type = strdup("wav");
       mfi->codectype = strdup("wav");
       mfi->description = strdup("PCM16 pipe");
@@ -662,22 +740,31 @@ filescanner_process_media(char *path, time_t mtime, off_t size, int type, struct
       goto out;
     }
 
-  if (type & F_SCAN_TYPE_COMPILATION)
-    mfi->compilation = 1;
-  if (type & F_SCAN_TYPE_PODCAST)
-    mfi->media_kind = 4; /* podcast */
-  if (type & F_SCAN_TYPE_AUDIOBOOK)
-    mfi->media_kind = 8; /* audiobook */
-
   if (!mfi->item_kind)
     mfi->item_kind = 2; /* music */
   if (!mfi->media_kind)
-    mfi->media_kind = 1; /* music */
+    mfi->media_kind = MEDIA_KIND_MUSIC; /* music */
 
   unicode_fixup_mfi(mfi);
 
   fixup_tags(mfi);
 
+  if (type & F_SCAN_TYPE_URL)
+    {
+      snprintf(virtual_path, PATH_MAX, "/http:/%s", mfi->title);
+      mfi->virtual_path = strdup(virtual_path);
+    }
+  else if (type & F_SCAN_TYPE_SPOTIFY)
+    {
+      snprintf(virtual_path, PATH_MAX, "/spotify:/%s/%s/%s", mfi->album_artist, mfi->album, mfi->title);
+      mfi->virtual_path = strdup(virtual_path);
+    }
+  else
+    {
+      snprintf(virtual_path, PATH_MAX, "/file:%s", mfi->path);
+      mfi->virtual_path = strdup(virtual_path);
+    }
+
   if (mfi->id == 0)
     db_file_add(mfi);
   else
@@ -758,12 +845,16 @@ process_deferred_playlists(void)
 static void
 process_file(char *file, time_t mtime, off_t size, int type, int flags)
 {
+  int is_bulkscan;
+
+  is_bulkscan = (flags & F_SCAN_BULK);
+
   switch (file_type_get(file))
     {
       case FILE_REGULAR:
 	filescanner_process_media(file, mtime, size, type, NULL);
 
-	cache_artwork_ping(file, mtime);
+	cache_artwork_ping(file, mtime, !is_bulkscan);
 	// TODO [artworkcache] If entry in artwork cache exists for no artwork available, delete the entry if media file has embedded artwork
 
 	counter++;
@@ -785,9 +876,14 @@ process_file(char *file, time_t mtime, off_t size, int type, int flags)
 	  process_playlist(file, mtime);
 	break;
 
+      case FILE_SMARTPL:
+	DPRINTF(E_DBG, L_SCAN, "Smart playlist file: %s\n", file);
+	scan_smartpl(file, mtime);
+	break;
+
       case FILE_ARTWORK:
 	DPRINTF(E_DBG, L_SCAN, "Artwork file: %s\n", file);
-	cache_artwork_ping(file, mtime);
+	cache_artwork_ping(file, mtime, !is_bulkscan);
 
 	// TODO [artworkcache] If entry in artwork cache exists for no artwork available for a album with files in the same directory, delete the entry
 
@@ -815,11 +911,7 @@ process_file(char *file, time_t mtime, off_t size, int type, int flags)
 
 	DPRINTF(E_LOG, L_SCAN, "Startup rescan triggered, found init-rescan file: %s\n", file);
 
-	inofd_event_unset(); // Clears all inotify watches
-	db_watch_clear();
-
-	inofd_event_set();
-	bulk_scan(F_SCAN_BULK | F_SCAN_RESCAN);
+	filescanner_initscan(NULL);
 	break;
 
       case FILE_CTRL_FULLSCAN:
@@ -828,13 +920,7 @@ process_file(char *file, time_t mtime, off_t size, int type, int flags)
 
 	DPRINTF(E_LOG, L_SCAN, "Full rescan triggered, found full-rescan file: %s\n", file);
 
-	player_playback_stop();
-	player_queue_clear();
-	inofd_event_unset(); // Clears all inotify watches
-	db_purge_all(); // Clears files, playlists, playlistitems, inotify and groups
-
-	inofd_event_set();
-	bulk_scan(F_SCAN_BULK);
+	filescanner_fullrescan(NULL);
 	break;
 
       default:
@@ -1064,6 +1150,9 @@ bulk_scan(int flags)
   time_t end;
   int i;
 
+  // Set global flag to avoid queued scan requests
+  scanning = 1;
+
   start = time(NULL);
 
   playlists = NULL;
@@ -1135,6 +1224,9 @@ bulk_scan(int flags)
       DPRINTF(E_DBG, L_SCAN, "Running post library scan jobs\n");
       db_hook_post_scan();
     }
+
+  // Set scan in progress flag to FALSE
+  scanning = 0;
 }
 
 
@@ -1201,7 +1293,7 @@ filescanner(void *arg)
 #endif
 
       /* Enable inotify */
-      event_add(&inoev, NULL);
+      event_add(inoev, NULL);
 
       event_base_dispatch(evbase_scan);
     }
@@ -1210,7 +1302,6 @@ filescanner(void *arg)
     DPRINTF(E_FATAL, L_SCAN, "Scan event loop terminated ahead of time!\n");
 
   db_perthread_deinit();
-  //artworkcache_perthread_deinit();
 
   pthread_exit(NULL);
 }
@@ -1386,13 +1477,17 @@ static void
 process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie)
 {
   struct stat sb;
+  uint32_t path_hash;
   char *deref = NULL;
   char *file = path;
   int type;
+  int i;
   int ret;
 
   DPRINTF(E_SPAM, L_SCAN, "File event: 0x%x, cookie 0x%x, wd %d\n", ie->mask, ie->cookie, wi->wd);
 
+  path_hash = djb_hash(path, strlen(path));
+
   if (ie->mask & IN_DELETE)
     {
       DPRINTF(E_DBG, L_SCAN, "File deleted: %s\n", path);
@@ -1412,7 +1507,14 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 
   if (ie->mask & IN_ATTRIB)
     {
-      DPRINTF(E_DBG, L_SCAN, "File permissions changed: %s\n", path);
+      DPRINTF(E_DBG, L_SCAN, "File attributes changed: %s\n", path);
+
+      // Ignore the IN_ATTRIB if we just got an IN_CREATE
+      for (i = 0; i < INCOMINGFILES_BUFFER_SIZE; i++)
+	{
+	  if (incomingfiles_buffer[i] == path_hash)
+	    return;
+	}
 
 #ifdef HAVE_EUIDACCESS
       if (euidaccess(path, R_OK) < 0)
@@ -1463,7 +1565,15 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 	  return;
 	}
 
-      if (S_ISFIFO(sb.st_mode))
+      // Add to the list of files where we ignore IN_ATTRIB until the file is closed again
+      if (S_ISREG(sb.st_mode))
+	{
+	  DPRINTF(E_SPAM, L_SCAN, "Incoming file created '%s' (%d), index %d\n", path, (int)path_hash, incomingfiles_idx);
+
+	  incomingfiles_buffer[incomingfiles_idx] = path_hash;
+	  incomingfiles_idx = (incomingfiles_idx + 1) % INCOMINGFILES_BUFFER_SIZE;
+	}
+      else if (S_ISFIFO(sb.st_mode))
 	ie->mask |= IN_CLOSE_WRITE;
     }
 
@@ -1471,6 +1581,15 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
     {
       DPRINTF(E_DBG, L_SCAN, "File closed: %s\n", path);
 
+      // File has been closed so remove from the IN_ATTRIB ignore list
+      for (i = 0; i < INCOMINGFILES_BUFFER_SIZE; i++)
+	if (incomingfiles_buffer[i] == path_hash)
+	  {
+	    DPRINTF(E_SPAM, L_SCAN, "Incoming file closed '%s' (%d), index %d\n", path, (int)path_hash, i);
+
+	    incomingfiles_buffer[i] = 0;
+	  }
+
       ret = lstat(path, &sb);
       if (ret < 0)
 	{
@@ -1635,7 +1754,7 @@ inotify_cb(int fd, short event, void *arg)
 
   free(buf);
 
-  event_add(&inoev, NULL);
+  event_add(inoev, NULL);
 }
 #endif /* __linux__ */
 
@@ -1818,7 +1937,7 @@ kqueue_cb(int fd, short event, void *arg)
 	DPRINTF(E_LOG, L_SCAN, "WARNING: unhandled leftover directories\n");
     }
 
-  event_add(&inoev, NULL);
+  event_add(inoev, NULL);
 }
 #endif /* __FreeBSD__ || __FreeBSD_kernel__ */
 
@@ -1835,7 +1954,7 @@ inofd_event_set(void)
       return -1;
     }
 
-  event_set(&inoev, inofd, EV_READ, inotify_cb, NULL);
+  inoev = event_new(evbase_scan, inofd, EV_READ, inotify_cb, NULL);
 
 #elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 
@@ -1847,11 +1966,9 @@ inofd_event_set(void)
       return -1;
     }
 
-  event_set(&inoev, inofd, EV_READ, kqueue_cb, NULL);
+  inoev = event_new(evbase_scan, inofd, EV_READ, kqueue_cb, NULL);
 #endif
 
-  event_base_set(evbase_scan, &inoev);
-
   return 0;
 }
 
@@ -1859,7 +1976,7 @@ inofd_event_set(void)
 static void
 inofd_event_unset(void)
 {
-  event_del(&inoev);
+  event_free(inoev);
   close(inofd);
 }
 
@@ -1872,6 +1989,134 @@ exit_cb(int fd, short event, void *arg)
   scan_exit = 1;
 }
 
+static void
+command_cb(int fd, short what, void *arg)
+{
+  struct filescanner_command *cmd;
+  int ret;
+
+  ret = read(cmd_pipe[0], &cmd, sizeof(cmd));
+  if (ret != sizeof(cmd))
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not read command! (read %d): %s\n", ret, (ret < 0) ? strerror(errno) : "-no error-");
+      goto readd;
+    }
+
+  if (cmd->nonblock)
+    {
+      cmd->func(cmd);
+
+      free(cmd);
+      goto readd;
+    }
+
+  pthread_mutex_lock(&cmd->lck);
+
+  ret = cmd->func(cmd);
+  cmd->ret = ret;
+
+  pthread_cond_signal(&cmd->cond);
+  pthread_mutex_unlock(&cmd->lck);
+
+ readd:
+  event_add(cmdev, NULL);
+}
+
+static int
+filescanner_initscan(struct filescanner_command *cmd)
+{
+  DPRINTF(E_LOG, L_SCAN, "Startup rescan triggered\n");
+
+  inofd_event_unset(); // Clears all inotify watches
+  db_watch_clear();
+
+  inofd_event_set();
+  bulk_scan(F_SCAN_BULK | F_SCAN_RESCAN);
+
+  return 0;
+}
+
+static int
+filescanner_fullrescan(struct filescanner_command *cmd)
+{
+  DPRINTF(E_LOG, L_SCAN, "Full rescan triggered\n");
+
+  player_playback_stop();
+  player_queue_clear();
+  inofd_event_unset(); // Clears all inotify watches
+  db_purge_all(); // Clears files, playlists, playlistitems, inotify and groups
+
+  inofd_event_set();
+  bulk_scan(F_SCAN_BULK);
+
+  return 0;
+}
+
+void
+filescanner_trigger_initscan(void)
+{
+  struct filescanner_command *cmd;
+
+  if (scanning)
+    {
+      DPRINTF(E_INFO, L_SCAN, "Scan already running, ignoring request to trigger a new init scan\n");
+      return;
+    }
+
+
+  cmd = (struct filescanner_command *)malloc(sizeof(struct filescanner_command));
+  if (!cmd)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not allocate cache_command\n");
+      return;
+    }
+
+  memset(cmd, 0, sizeof(struct filescanner_command));
+
+  cmd->nonblock = 1;
+
+  cmd->func = filescanner_initscan;
+
+  nonblock_command(cmd);
+}
+
+void
+filescanner_trigger_fullrescan(void)
+{
+  struct filescanner_command *cmd;
+
+  if (scanning)
+    {
+      DPRINTF(E_INFO, L_SCAN, "Scan already running, ignoring request to trigger a new init scan\n");
+      return;
+    }
+
+  cmd = (struct filescanner_command *)malloc(sizeof(struct filescanner_command));
+  if (!cmd)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not allocate cache_command\n");
+      return;
+    }
+
+  memset(cmd, 0, sizeof(struct filescanner_command));
+
+  cmd->nonblock = 1;
+
+  cmd->func = filescanner_fullrescan;
+
+  nonblock_command(cmd);
+}
+
+/*
+ * Query the status of the filescanner
+ * @return 1 if scan is running, otherwise 0
+ */
+int
+filescanner_scanning(void)
+{
+  return scanning;
+}
+
 /* Thread: main */
 int
 filescanner_init(void)
@@ -1879,6 +2124,7 @@ filescanner_init(void)
   int ret;
 
   scan_exit = 0;
+  scanning = 0;
 
   evbase_scan = event_base_new();
   if (!evbase_scan)
@@ -1888,27 +2134,20 @@ filescanner_init(void)
       return -1;
     }
 
-#ifdef USE_EVENTFD
-  exit_efd = eventfd(0, EFD_CLOEXEC);
-  if (exit_efd < 0)
-    {
-      DPRINTF(E_FATAL, L_SCAN, "Could not create eventfd: %s\n", strerror(errno));
-
-      goto pipe_fail;
-    }
-#else
-# if defined(__linux__)
   ret = pipe2(exit_pipe, O_CLOEXEC);
-# else
-  ret = pipe(exit_pipe);
-# endif
   if (ret < 0)
     {
       DPRINTF(E_FATAL, L_SCAN, "Could not create pipe: %s\n", strerror(errno));
 
       goto pipe_fail;
     }
-#endif /* USE_EVENTFD */
+
+  exitev = event_new(evbase_scan, exit_pipe[0], EV_READ, exit_cb, NULL);
+  if (!exitev || (event_add(exitev, NULL) < 0))
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not create/add command event\n");
+      goto exitev_fail;
+    }
 
   ret = inofd_event_set();
   if (ret < 0)
@@ -1916,13 +2155,19 @@ filescanner_init(void)
       goto ino_fail;
     }
 
-#ifdef USE_EVENTFD
-  event_set(&exitev, exit_efd, EV_READ, exit_cb, NULL);
-#else
-  event_set(&exitev, exit_pipe[0], EV_READ, exit_cb, NULL);
-#endif
-  event_base_set(evbase_scan, &exitev);
-  event_add(&exitev, NULL);
+  ret = pipe2(cmd_pipe, O_CLOEXEC);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not create command pipe: %s\n", strerror(errno));
+      goto cmd_fail;
+    }
+
+  cmdev = event_new(evbase_scan, cmd_pipe[0], EV_READ, command_cb, NULL);
+  if (!cmdev || (event_add(cmdev, NULL) < 0))
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not create/add command event\n");
+      goto cmd_fail;
+    }
 
   ret = pthread_create(&tid_scan, NULL, filescanner, NULL);
   if (ret != 0)
@@ -1935,14 +2180,14 @@ filescanner_init(void)
   return 0;
 
  thread_fail:
+ cmd_fail:
+  close(cmd_pipe[0]);
+  close(cmd_pipe[1]);
   close(inofd);
+ exitev_fail:
  ino_fail:
-#ifdef USE_EVENTFD
-  close(exit_efd);
-#else
   close(exit_pipe[0]);
   close(exit_pipe[1]);
-#endif
  pipe_fail:
   event_base_free(evbase_scan);
 
@@ -1954,16 +2199,6 @@ void
 filescanner_deinit(void)
 {
   int ret;
-
-#ifdef USE_EVENTFD
-  ret = eventfd_write(exit_efd, 1);
-  if (ret < 0)
-    {
-      DPRINTF(E_FATAL, L_SCAN, "Could not send exit event: %s\n", strerror(errno));
-
-      return;
-    }
-#else
   int dummy = 42;
 
   ret = write(exit_pipe[1], &dummy, sizeof(dummy));
@@ -1973,7 +2208,6 @@ filescanner_deinit(void)
 
       return;
     }
-#endif
 
   scan_exit = 1;
 
@@ -1987,11 +2221,9 @@ filescanner_deinit(void)
 
   inofd_event_unset();
 
-#ifdef USE_EVENTFD
-  close(exit_efd);
-#else
   close(exit_pipe[0]);
   close(exit_pipe[1]);
-#endif
+  close(cmd_pipe[0]);
+  close(cmd_pipe[1]);
   event_base_free(evbase_scan);
 }
