@@ -40,7 +40,6 @@
 #include <uninorm.h>
 #include <unistd.h>
 
-#include "avl/avl.h"
 #include "logger.h"
 #include "db.h"
 #include "conffile.h"
@@ -53,22 +52,30 @@
 #include "dmap_common.h"
 #include "cache.h"
 
-#ifdef HAVE_LIBEVENT2
-# include <event2/http_struct.h>
-#endif
+#include <event2/event.h>
+#include <event2/buffer.h>
+#include <event2/http_struct.h>
+#include <event2/keyvalq_struct.h>
 
 /* httpd event base, from httpd.c */
 extern struct event_base *evbase_httpd;
 
-/* First session id to use */
-#define DAAP_BASE_SESSION_ID 100
-/* Session timeout in seconds */
-#define DAAP_SESSION_TIMEOUT 0 /* By default the session never times out */
+/* Max number of sessions and session timeout
+ * Many clients (including iTunes) don't seem to respect the timeout capability
+ * that we announce, and just keep using the same session. Therefore we take a
+ * lenient approach to actually timing out: We wait an entire week, and to
+ * avoid running a timer for that long, we only check for expiration when adding
+ * new sessions - see daap_session_cleanup().
+ */
+#define DAAP_SESSION_MAX 200
+#define DAAP_SESSION_TIMEOUT 604800            // One week in seconds
 /* We announce this timeout to the client when returning server capabilities */
-#define DAAP_SESSION_TIMEOUT_CAPABILITY 1800 
+#define DAAP_SESSION_TIMEOUT_CAPABILITY 1800   // 30 minutes
 /* Update requests refresh interval in seconds */
 #define DAAP_UPDATE_REFRESH  0
 
+/* Database number for the Radio item */
+#define DAAP_DB_RADIO 2
 
 struct uri_map {
   regex_t preg;
@@ -79,14 +86,16 @@ struct uri_map {
 struct daap_session {
   int id;
   char *user_agent;
-  struct event timeout;
+  time_t mtime;
+
+  struct daap_session *next;
 };
 
 struct daap_update_request {
   struct evhttp_request *req;
 
   /* Refresh tiemout */
-  struct event timeout;
+  struct event *timeout;
 
   struct daap_update_request *next;
 };
@@ -106,40 +115,18 @@ static char *default_meta_pl = "dmap.itemid,dmap.itemname,dmap.persistentid,com.
 static char *default_meta_group = "dmap.itemname,dmap.persistentid,daap.songalbumartist";
 
 /* DAAP session tracking */
-static avl_tree_t *daap_sessions;
-static int next_session_id;
+static struct daap_session *daap_sessions;
 
 /* Update requests */
 static int current_rev;
 static struct daap_update_request *update_requests;
+static struct timeval daap_update_refresh_tv = { DAAP_UPDATE_REFRESH, 0 };
 
 
 /* Session handling */
-static int
-daap_session_compare(const void *aa, const void *bb)
-{
-  struct daap_session *a = (struct daap_session *)aa;
-  struct daap_session *b = (struct daap_session *)bb;
-
-  if (a->id < b->id)
-    return -1;
-
-  if (a->id > b->id)
-    return 1;
-
-  return 0;
-}
-
 static void
-daap_session_free(void *item)
+daap_session_free(struct daap_session *s)
 {
-  struct daap_session *s;
-
-  s = (struct daap_session *)item;
-
-  if (event_initialized(&s->timeout))
-    evtimer_del(&s->timeout);
-
   if (s->user_agent)
     free(s->user_agent);
 
@@ -147,30 +134,82 @@ daap_session_free(void *item)
 }
 
 static void
-daap_session_kill(struct daap_session *s)
+daap_session_remove(struct daap_session *s)
 {
-  avl_delete(daap_sessions, s);
-}
+  struct daap_session *ptr;
+  struct daap_session *prev;
 
-static void
-daap_session_timeout_cb(int fd, short what, void *arg)
-{
-  struct daap_session *s;
+  prev = NULL;
+  for (ptr = daap_sessions; ptr; ptr = ptr->next)
+    {
+      if (ptr == s)
+	break;
 
-  s = (struct daap_session *)arg;
+      prev = ptr;
+    }
 
-  DPRINTF(E_DBG, L_DAAP, "Session %d timed out\n", s->id);
+  if (!ptr)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Error: Request to remove non-existent session. BUG!\n");
+      return;
+    }
 
-  daap_session_kill(s);
+  if (!prev)
+    daap_sessions = s->next;
+  else
+    prev->next = s->next;
+
+  daap_session_free(s);
 }
 
 static struct daap_session *
-daap_session_register(const char *user_agent, int request_session_id)
+daap_session_get(int id)
 {
-  struct timeval tv;
   struct daap_session *s;
-  avl_node_t *node;
-  int ret;
+
+  for (s = daap_sessions; s; s = s->next)
+    {
+      if (id == s->id)
+	return s;
+    }
+
+  return NULL;
+}
+
+/* Removes stale sessions and also drops the oldest sessions if DAAP_SESSION_MAX
+ * will otherwise be exceeded
+ */
+static void
+daap_session_cleanup(void)
+{
+  struct daap_session *s;
+  struct daap_session *next;
+  time_t now;
+  int count;
+
+  count = 0;
+  now = time(NULL);
+
+  for (s = daap_sessions; s; s = next)
+    {
+      count++;
+      next = s->next;
+
+      if ((difftime(now, s->mtime) > DAAP_SESSION_TIMEOUT) || (count > DAAP_SESSION_MAX))
+	{
+	  DPRINTF(E_LOG, L_DAAP, "Cleaning up DAAP session (id %d)\n", s->id);
+
+	  daap_session_remove(s);
+	}
+    }
+}
+
+static struct daap_session *
+daap_session_add(const char *user_agent, int request_session_id)
+{
+  struct daap_session *s;
+
+  daap_session_cleanup();
 
   s = (struct daap_session *)malloc(sizeof(struct daap_session));
   if (!s)
@@ -181,54 +220,30 @@ daap_session_register(const char *user_agent, int request_session_id)
 
   memset(s, 0, sizeof(struct daap_session));
 
-  if (!request_session_id)
+  if (request_session_id)
     {
-      s->id = next_session_id;
-      next_session_id++;
-    }
-  else if (request_session_id < DAAP_BASE_SESSION_ID)
-    {
-      s->id = request_session_id;
-      if (avl_search(daap_sessions, s))
+      if (daap_session_get(request_session_id))
 	{
 	  DPRINTF(E_LOG, L_DAAP, "Session id requested in login (%d) is not available\n", request_session_id);
 	  return NULL;
 	}
-    }    
+      
+      s->id = request_session_id;
+    }
   else
     {
-      DPRINTF(E_LOG, L_DAAP, "Session id requested in login (%d) must be below %d\n", request_session_id, DAAP_BASE_SESSION_ID);
-      return NULL;
+      while ( (s->id = rand() + 100) && daap_session_get(s->id) );
     }
 
+  s->mtime = time(NULL);
+
   if (user_agent)
     s->user_agent = strdup(user_agent);
 
-  if (DAAP_SESSION_TIMEOUT > 0) {
-    evtimer_set(&s->timeout, daap_session_timeout_cb, s);
-    event_base_set(evbase_httpd, &s->timeout);
-  }
+  if (daap_sessions)
+    s->next = daap_sessions;
 
-  node = avl_insert(daap_sessions, s);
-  if (!node)
-    {
-      DPRINTF(E_LOG, L_DAAP, "Could not register DAAP session: %s\n", strerror(errno));
-
-      if (user_agent)
-	free(s->user_agent);
-
-      free(s);
-      return NULL;
-    }
-
-  if (DAAP_SESSION_TIMEOUT > 0) {
-    evutil_timerclear(&tv);
-    tv.tv_sec = DAAP_SESSION_TIMEOUT;
-    
-    ret = evtimer_add(&s->timeout, &tv);
-    if (ret < 0)
-      DPRINTF(E_LOG, L_DAAP, "Could not add session timeout event for session %d\n", s->id);
-  }
+  daap_sessions = s;
 
   return s;
 }
@@ -236,11 +251,9 @@ daap_session_register(const char *user_agent, int request_session_id)
 struct daap_session *
 daap_session_find(struct evhttp_request *req, struct evkeyvalq *query, struct evbuffer *evbuf)
 {
-  struct daap_session needle;
-  struct timeval tv;
   struct daap_session *s;
-  avl_node_t *node;
   const char *param;
+  int id;
   int ret;
 
   if (!req)
@@ -253,31 +266,18 @@ daap_session_find(struct evhttp_request *req, struct evkeyvalq *query, struct ev
       goto invalid;
     }
 
-  ret = safe_atoi32(param, &needle.id);
+  ret = safe_atoi32(param, &id);
   if (ret < 0)
     goto invalid;
 
-  node = avl_search(daap_sessions, &needle);
-  if (!node)
+  s = daap_session_get(id);
+  if (!s)
     {
-      DPRINTF(E_WARN, L_DAAP, "DAAP session id %d not found\n", needle.id);
+      DPRINTF(E_LOG, L_DAAP, "DAAP session id %d not found\n", id);
       goto invalid;
     }
 
-  s = (struct daap_session *)node->item;
-
-  if (DAAP_SESSION_TIMEOUT > 0) {
-    event_del(&s->timeout);
-  }
-
-  evutil_timerclear(&tv);
-  tv.tv_sec = DAAP_SESSION_TIMEOUT;
-
-  if (DAAP_SESSION_TIMEOUT > 0) {
-    ret = evtimer_add(&s->timeout, &tv);
-    if (ret < 0)
-      DPRINTF(E_LOG, L_DAAP, "Could not add session timeout event for session %d\n", s->id);
-  }
+  s->mtime = time(NULL);
 
   return s;
 
@@ -288,6 +288,15 @@ daap_session_find(struct evhttp_request *req, struct evkeyvalq *query, struct ev
 
 
 /* Update requests helpers */
+static void
+update_free(struct daap_update_request *ur)
+{
+  if (ur->timeout)
+    event_free(ur->timeout);
+
+  free(ur);
+}
+
 static void
 update_remove(struct daap_update_request *ur)
 {
@@ -308,14 +317,8 @@ update_remove(struct daap_update_request *ur)
 
       p->next = ur->next;
     }
-}
 
-static void
-update_free(struct daap_update_request *ur)
-{
-  if (event_initialized(&ur->timeout))
-    evtimer_del(&ur->timeout);
-  free(ur);
+  update_free(ur);
 }
 
 static void
@@ -355,7 +358,6 @@ update_refresh_cb(int fd, short event, void *arg)
   httpd_send_reply(ur->req, HTTP_OK, "OK", evbuf);
 
   update_remove(ur);
-  update_free(ur);
 }
 
 static void
@@ -373,7 +375,6 @@ update_fail_cb(struct evhttp_connection *evcon, void *arg)
     evhttp_connection_set_closecb(evc, NULL, NULL);
 
   update_remove(ur);
-  update_free(ur);
 }
 
 
@@ -435,7 +436,7 @@ daap_sort_build(struct sort_ctx *ctx, char *str)
   len = strlen(str);
   if (len > 0)
     {
-      ret = u8_normalize(UNINORM_NFD, (uint8_t *)str, len, NULL, &len);
+      ret = u8_normalize(UNINORM_NFD, (uint8_t *)str, len + 1, NULL, &len);
       if (!ret)
 	{
 	  DPRINTF(E_LOG, L_DAAP, "Could not normalize string for sort header\n");
@@ -684,7 +685,11 @@ get_query_params(struct evkeyvalq *query, int *sort_headers, struct query_params
 
       qp->filter = daap_query_parse_sql(param);
       if (!qp->filter)
-	DPRINTF(E_LOG, L_DAAP, "Ignoring improper DAAP query\n");
+	DPRINTF(E_LOG, L_DAAP, "Ignoring improper DAAP query: %s\n", param);
+
+      /* iTunes seems to default to this when there is a query (which there is for audiobooks, but not for normal playlists) */
+      if (qp->sort == S_NONE)
+	qp->sort = S_ALBUM;
     }
 }
 
@@ -778,6 +783,7 @@ daap_reply_server_info(struct evhttp_request *req, struct evbuffer *evbuf, char
   char *name;
   char *passwd;
   const char *clientver;
+  size_t len;
   int mpro;
   int apro;
 
@@ -862,13 +868,14 @@ daap_reply_server_info(struct evhttp_request *req, struct evbuffer *evbuf, char
   dmap_add_char(content, "msix", 1);         // dmap.supportsindex
 //  dmap_add_char(content, "msrs", 1);       // dmap.supportsresolve
 
-  dmap_add_int(content, "msdc", 1);          // dmap.databasescount
+  dmap_add_int(content, "msdc", 2);          // dmap.databasescount
 
 //  dmap_add_int(content, "mstc", );          // dmap.utctime
 //  dmap_add_int(content, "msto", );          // dmap.utcoffset
 
   // Create container
-  dmap_add_container(evbuf, "msrv", EVBUFFER_LENGTH(content));
+  len = evbuffer_get_length(content);
+  dmap_add_container(evbuf, "msrv", len);
   evbuffer_add_buffer(evbuf, content);
   evbuffer_free(content);
 
@@ -881,9 +888,9 @@ static int
 daap_reply_content_codes(struct evhttp_request *req, struct evbuffer *evbuf, char **uri, struct evkeyvalq *query, const char *ua)
 {
   const struct dmap_field *dmap_fields;
+  size_t len;
   int nfields;
   int i;
-  int len;
   int ret;
 
   dmap_fields = dmap_get_fields_table(&nfields);
@@ -978,7 +985,7 @@ daap_reply_login(struct evhttp_request *req, struct evbuffer *evbuf, char **uri,
   else
     request_session_id = 0;
 
-  s = daap_session_register(ua, request_session_id);
+  s = daap_session_add(ua, request_session_id);
   if (!s)
     {
       dmap_send_error(req, "mlog", "Could not start session");
@@ -1003,7 +1010,7 @@ daap_reply_logout(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
   if (!s)
     return -1;
 
-  daap_session_kill(s);
+  daap_session_remove(s);
 
   httpd_send_reply(req, 204, "Logout Successful", evbuf);
 
@@ -1013,7 +1020,6 @@ daap_reply_logout(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
 static int
 daap_reply_update(struct evhttp_request *req, struct evbuffer *evbuf, char **uri, struct evkeyvalq *query, const char *ua)
 {
-  struct timeval tv;
   struct daap_session *s;
   struct daap_update_request *ur;
   struct evhttp_connection *evcon;
@@ -1075,24 +1081,23 @@ daap_reply_update(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
     }
   memset(ur, 0, sizeof(struct daap_update_request));
 
-  if (DAAP_UPDATE_REFRESH > 0) {
-    evtimer_set(&ur->timeout, update_refresh_cb, ur);
-    event_base_set(evbase_httpd, &ur->timeout);
-    
-    evutil_timerclear(&tv);
-    tv.tv_sec = DAAP_UPDATE_REFRESH;
-    
-    ret = evtimer_add(&ur->timeout, &tv);
-    if (ret < 0)
-      {
-	DPRINTF(E_LOG, L_DAAP, "Could not add update timeout event\n");
-	
-	dmap_send_error(req, "mupd", "Could not register timer");
-	
-	update_free(ur);
-	return -1;
-      }
-  }
+  if (DAAP_UPDATE_REFRESH > 0)
+    {
+      ur->timeout = evtimer_new(evbase_httpd, update_refresh_cb, ur);
+      if (ur->timeout)
+	ret = evtimer_add(ur->timeout, &daap_update_refresh_tv);
+      else
+	ret = -1;
+
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_DAAP, "Out of memory for update request event\n");
+
+	  dmap_send_error(req, "mupd", "Could not register timer");	
+	  update_free(ur);
+	  return -1;
+	}
+    }
 
   /* NOTE: we may need to keep reqd_rev in there too */
   ur->req = req;
@@ -1123,9 +1128,12 @@ static int
 daap_reply_dblist(struct evhttp_request *req, struct evbuffer *evbuf, char **uri, struct evkeyvalq *query, const char *ua)
 {
   struct evbuffer *content;
+  struct evbuffer *item;
   struct daap_session *s;
   cfg_t *lib;
   char *name;
+  char *name_radio;
+  size_t len;
   int count;
 
   s = daap_session_find(req, query, evbuf);
@@ -1134,6 +1142,7 @@ daap_reply_dblist(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
 
   lib = cfg_getsec(cfg, "library");
   name = cfg_getstr(lib, "name");
+  name_radio = cfg_getstr(lib, "name_radio");
 
   content = evbuffer_new();
   if (!content)
@@ -1144,29 +1153,69 @@ daap_reply_dblist(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
       return -1;
     }
 
-  dmap_add_int(content, "miid", 1);
-  dmap_add_long(content, "mper", 1);
-  dmap_add_int(content, "mdbk", 1);
-  dmap_add_int(content, "aeCs", 1);
-  dmap_add_string(content, "minm", name);
+  // Add db entry for library with dbid = 1
+  item = evbuffer_new();
+  if (!item)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not create evbuffer for DAAP dblist library item\n");
 
+      dmap_send_error(req, "avdb", "Out of memory");
+      return -1;
+    }
+
+  dmap_add_int(item, "miid", 1);
+  dmap_add_long(item, "mper", 1);
+  dmap_add_int(item, "mdbk", 1);
+  dmap_add_int(item, "aeCs", 1);
+  dmap_add_string(item, "minm", name);
   count = db_files_get_count();
-  dmap_add_int(content, "mimc", count);
-
+  dmap_add_int(item, "mimc", count);
   count = db_pl_get_count(); // TODO Don't count empty smart playlists, because they get excluded in aply
-  dmap_add_int(content, "mctc", count);
-
+  dmap_add_int(item, "mctc", count);
 //  dmap_add_int(content, "aeMk", 0x405);   // com.apple.itunes.extended-media-kind (OR of all in library)
-  dmap_add_int(content, "meds", 3);
+  dmap_add_int(item, "meds", 3);
+
+  // Create container for library db
+  len = evbuffer_get_length(item);
+  dmap_add_container(content, "mlit", len);
+  evbuffer_add_buffer(content, item);
+  evbuffer_free(item);
+
+  // Add second db entry for radio with dbid = DAAP_DB_RADIO
+  item =  evbuffer_new();
+  if (!item)
+    {
+      DPRINTF(E_LOG, L_DAAP, "Could not create evbuffer for DAAP dblist radio item\n");
+
+      dmap_send_error(req, "avdb", "Out of memory");
+      return -1;
+    }
+
+  dmap_add_int(item, "miid", DAAP_DB_RADIO);
+  dmap_add_long(item, "mper", DAAP_DB_RADIO);
+  dmap_add_int(item, "mdbk", 0x64);
+  dmap_add_int(item, "aeCs", 0);
+  dmap_add_string(item, "minm", name_radio);
+  count = db_pl_get_count();       // TODO This counts too much, should only include stream playlists
+  dmap_add_int(item, "mimc", count);
+  dmap_add_int(item, "mctc", 0);
+  dmap_add_int(item, "aeMk", 1);   // com.apple.itunes.extended-media-kind (OR of all in library)
+  dmap_add_int(item, "meds", 3);
+
+  // Create container for radio db
+  len = evbuffer_get_length(item);
+  dmap_add_container(content, "mlit", len);
+  evbuffer_add_buffer(content, item);
+  evbuffer_free(item);
 
   // Create container
-  dmap_add_container(evbuf, "avdb", EVBUFFER_LENGTH(content) + 61);
+  len = evbuffer_get_length(content);
+  dmap_add_container(evbuf, "avdb", len + 53);
   dmap_add_int(evbuf, "mstt", 200);     /* 12 */
   dmap_add_char(evbuf, "muty", 0);      /* 9 */
-  dmap_add_int(evbuf, "mtco", 1);       /* 12 */
-  dmap_add_int(evbuf, "mrco", 1);       /* 12 */
-  dmap_add_container(evbuf, "mlcl", EVBUFFER_LENGTH(content) + 8); /* 8 */
-  dmap_add_container(evbuf, "mlit", EVBUFFER_LENGTH(content));     /* 8 */
+  dmap_add_int(evbuf, "mtco", 2);       /* 12 */
+  dmap_add_int(evbuf, "mrco", 2);       /* 12 */
+  dmap_add_container(evbuf, "mlcl", len); /* 8 */
   evbuffer_add_buffer(evbuf, content);
   evbuffer_free(content);
 
@@ -1190,6 +1239,7 @@ daap_reply_songlist_generic(struct evhttp_request *req, struct evbuffer *evbuf,
   const char *client_codecs;
   char *last_codectype;
   char *tag;
+  size_t len;
   int nmeta;
   int sort_headers;
   int nsongs;
@@ -1411,18 +1461,19 @@ daap_reply_songlist_generic(struct evhttp_request *req, struct evbuffer *evbuf,
     }
 
   /* Add header to evbuf, add songlist to evbuf */
+  len = evbuffer_get_length(songlist);
   if (sort_headers)
     {
       daap_sort_finalize(sctx);
-      dmap_add_container(evbuf, tag, EVBUFFER_LENGTH(songlist) + EVBUFFER_LENGTH(sctx->headerlist) + 61);
+      dmap_add_container(evbuf, tag, len + evbuffer_get_length(sctx->headerlist) + 61);
     }
   else
-    dmap_add_container(evbuf, tag, EVBUFFER_LENGTH(songlist) + 53);
+    dmap_add_container(evbuf, tag, len + 53);
   dmap_add_int(evbuf, "mstt", 200);    /* 12 */
   dmap_add_char(evbuf, "muty", 0);     /* 9 */
   dmap_add_int(evbuf, "mtco", qp.results); /* 12 */
   dmap_add_int(evbuf, "mrco", nsongs); /* 12 */
-  dmap_add_container(evbuf, "mlcl", EVBUFFER_LENGTH(songlist)); /* 8 */
+  dmap_add_container(evbuf, "mlcl", len); /* 8 */
 
   db_query_end(&qp);
 
@@ -1442,7 +1493,8 @@ daap_reply_songlist_generic(struct evhttp_request *req, struct evbuffer *evbuf,
 
   if (sort_headers)
     {
-      dmap_add_container(evbuf, "mshl", EVBUFFER_LENGTH(sctx->headerlist)); /* 8 */
+      len = evbuffer_get_length(sctx->headerlist);
+      dmap_add_container(evbuf, "mshl", len); /* 8 */
       ret = evbuffer_add_buffer(evbuf, sctx->headerlist);
       daap_sort_context_free(sctx);
 
@@ -1506,16 +1558,22 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
   struct daap_session *s;
   struct evbuffer *playlistlist;
   struct evbuffer *playlist;
+  cfg_t *lib;
   const struct dmap_field_map *dfm;
   const struct dmap_field *df;
   const struct dmap_field **meta;
   const char *param;
   char **strval;
+  size_t len;
+  int database;
+  int cfg_radiopl;
   int nmeta;
   int npls;
   int32_t plid;
   int32_t pltype;
   int32_t plitems;
+  int32_t plstreams;
+  int32_t plparent;
   int i;
   int ret;
 
@@ -1523,6 +1581,17 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
   if (!s)
     return -1;
 
+  ret = safe_atoi32(uri[1], &database);
+  if (ret < 0)
+    {
+      dmap_send_error(req, "aply", "Invalid database ID");
+
+      return -1;
+    }
+
+  lib = cfg_getsec(cfg, "library");
+  cfg_radiopl = cfg_getbool(lib, "radio_playlists");
+
   ret = evbuffer_expand(evbuf, 61);
   if (ret < 0)
     {
@@ -1615,8 +1684,20 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
       if (safe_atoi32(dbpli.items, &plitems) != 0)
 	continue;
 
-      /* Don't add empty smart playlists */
-      if ((plid > 1) && (pltype == 1) && (plitems == 0))
+      plstreams = 0;
+      if (safe_atoi32(dbpli.streams, &plstreams) != 0)
+	continue;
+
+      /* Database DAAP_DB_RADIO is radio, so for that db skip playlists without
+       * streams and for other databases skip playlists which are just streams
+       */
+      if ((database == DAAP_DB_RADIO) && (plstreams == 0))
+	continue;
+      if (!cfg_radiopl && (database != DAAP_DB_RADIO) && (plstreams > 0) && (plstreams == plitems))
+	continue;
+
+      /* Don't add empty Special playlists */
+      if ((plid > 1) && (plitems == 0) && (pltype == PL_SPECIAL))
 	continue;
 
       npls++;
@@ -1630,14 +1711,20 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
 	  if (dfm == &dfm_dmap_mimc)
 	    continue;
 
-	  /* com.apple.itunes.smart-playlist - type = 1 AND id != 1 */
+	  /* Add field "com.apple.itunes.smart-playlist" for special and smart playlists
+	     (excluding the special playlist for "library" with id = 1) */
 	  if (dfm == &dfm_dmap_aeSP)
 	    {
-	      if ((pltype == PL_SMART) && (plid != 1))
+	      if ((pltype == PL_SMART) || ((pltype == PL_SPECIAL) && (plid != 1)))
+		{
+		  dmap_add_char(playlist, "aeSP", 1);
+		}
+
+	      /* Add field "com.apple.itunes.special-playlist" for special playlists
+		 (excluding the special playlist for "library" with id = 1) */
+	      if ((pltype == PL_SPECIAL) && (plid != 1))
 		{
 		  int32_t aePS = 0;
-		  dmap_add_char(playlist, "aeSP", 1);
-
 		  ret = safe_atoi32(dbpli.special_id, &aePS);
 		  if ((ret == 0) && (aePS > 0))
 		    dmap_add_char(playlist, "aePS", aePS);
@@ -1661,11 +1748,14 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
 	}
 
       /* Item count (mimc) */
-      if (plitems > 0)
-	dmap_add_int(playlist, "mimc", plitems);
+      dmap_add_int(playlist, "mimc", plitems);
 
       /* Container ID (mpco) */
-      dmap_add_int(playlist, "mpco", 0);
+      ret = safe_atoi32(dbpli.parent_id, &plparent);
+      if (ret == 0)
+	dmap_add_int(playlist, "mpco", plparent);
+      else
+	dmap_add_int(playlist, "mpco", 0);
 
       /* Base playlist (abpl), id = 1 */
       if (plid == 1)
@@ -1673,7 +1763,8 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
 
       DPRINTF(E_DBG, L_DAAP, "Done with playlist\n");
 
-      dmap_add_container(playlistlist, "mlit", EVBUFFER_LENGTH(playlist));
+      len = evbuffer_get_length(playlist);
+      dmap_add_container(playlistlist, "mlit", len);
       ret = evbuffer_add_buffer(playlistlist, playlist);
       if (ret < 0)
 	{
@@ -1707,12 +1798,13 @@ daap_reply_playlists(struct evhttp_request *req, struct evbuffer *evbuf, char **
     }
 
   /* Add header to evbuf, add playlistlist to evbuf */
-  dmap_add_container(evbuf, "aply", EVBUFFER_LENGTH(playlistlist) + 53);
+  len = evbuffer_get_length(playlistlist);
+  dmap_add_container(evbuf, "aply", len + 53);
   dmap_add_int(evbuf, "mstt", 200); /* 12 */
   dmap_add_char(evbuf, "muty", 0);  /* 9 */
   dmap_add_int(evbuf, "mtco", qp.results); /* 12 */
   dmap_add_int(evbuf,"mrco", npls); /* 12 */
-  dmap_add_container(evbuf, "mlcl", EVBUFFER_LENGTH(playlistlist));
+  dmap_add_container(evbuf, "mlcl", len);
 
   db_query_end(&qp);
 
@@ -1756,15 +1848,17 @@ daap_reply_groups(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
   const struct dmap_field *df;
   const struct dmap_field **meta;
   struct sort_ctx *sctx;
+  cfg_t *lib;
   const char *param;
   char **strval;
+  char *tag;
+  size_t len;
   int nmeta;
   int sort_headers;
   int ngrp;
   int32_t val;
   int i;
   int ret;
-  char *tag;
 
   s = daap_session_find(req, query, evbuf);
   if (!s && req)
@@ -1886,6 +1980,11 @@ daap_reply_groups(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
       if (strlen(dbgri.itemname) == 0)
 	continue;
 
+      /* Don't add single item albums/artists if configured to hide */
+      lib = cfg_getsec(cfg, "library");
+      if (cfg_getbool(lib, "hide_singles") && (strcmp(dbgri.itemcount, "1") == 0))
+	continue;
+
       ngrp++;
 
       for (i = 0; i < nmeta; i++)
@@ -1939,9 +2038,10 @@ daap_reply_groups(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
       if ((ret == 0) && (val > 0))
 	dmap_add_int(group, "miid", val);
 
-      DPRINTF(E_DBG, L_DAAP, "Done with group\n");
+      DPRINTF(E_SPAM, L_DAAP, "Done with group\n");
 
-      dmap_add_container(grouplist, "mlit", EVBUFFER_LENGTH(group));
+      len = evbuffer_get_length(group);
+      dmap_add_container(grouplist, "mlit", len);
       ret = evbuffer_add_buffer(grouplist, group);
       if (ret < 0)
 	{
@@ -1982,19 +2082,20 @@ daap_reply_groups(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
     }
 
   /* Add header to evbuf, add grouplist to evbuf */
+  len = evbuffer_get_length(grouplist);
   if (sort_headers)
     {
       daap_sort_finalize(sctx);
-      dmap_add_container(evbuf, tag, EVBUFFER_LENGTH(grouplist) + EVBUFFER_LENGTH(sctx->headerlist) + 61);
+      dmap_add_container(evbuf, tag, len + evbuffer_get_length(sctx->headerlist) + 61);
     }
   else
-    dmap_add_container(evbuf, tag, EVBUFFER_LENGTH(grouplist) + 53);
+    dmap_add_container(evbuf, tag, len + 53);
 
   dmap_add_int(evbuf, "mstt", 200); /* 12 */
   dmap_add_char(evbuf, "muty", 0);  /* 9 */
   dmap_add_int(evbuf, "mtco", qp.results); /* 12 */
   dmap_add_int(evbuf,"mrco", ngrp); /* 12 */
-  dmap_add_container(evbuf, "mlcl", EVBUFFER_LENGTH(grouplist)); /* 8 */
+  dmap_add_container(evbuf, "mlcl", len); /* 8 */
 
   db_query_end(&qp);
 
@@ -2014,7 +2115,8 @@ daap_reply_groups(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
 
   if (sort_headers)
     {
-      dmap_add_container(evbuf, "mshl", EVBUFFER_LENGTH(sctx->headerlist)); /* 8 */
+      len = evbuffer_get_length(sctx->headerlist);
+      dmap_add_container(evbuf, "mshl", len); /* 8 */
       ret = evbuffer_add_buffer(evbuf, sctx->headerlist);
       daap_sort_context_free(sctx);
 
@@ -2056,6 +2158,7 @@ daap_reply_browse(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
   char *browse_item;
   char *sort_item;
   char *tag;
+  size_t len;
   int sort_headers;
   int nitems;
   int ret;
@@ -2086,11 +2189,13 @@ daap_reply_browse(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
     {
       tag = "abgn";
       qp.type = Q_BROWSE_GENRES;
+      qp.sort = S_GENRE;
     }
   else if (strcmp(uri[3], "composers") == 0)
     {
       tag = "abcp";
       qp.type = Q_BROWSE_COMPOSERS;
+      qp.sort = S_COMPOSER;
     }
   else
     {
@@ -2187,19 +2292,20 @@ daap_reply_browse(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
       goto out_sort_headers_free;
     }
 
+  len = evbuffer_get_length(itemlist);
   if (sort_headers)
     {
       daap_sort_finalize(sctx);
-      dmap_add_container(evbuf, "abro", EVBUFFER_LENGTH(itemlist) + EVBUFFER_LENGTH(sctx->headerlist) + 52);
+      dmap_add_container(evbuf, "abro", len + evbuffer_get_length(sctx->headerlist) + 52);
     }
   else
-    dmap_add_container(evbuf, "abro", EVBUFFER_LENGTH(itemlist) + 44);
+    dmap_add_container(evbuf, "abro", len + 44);
 
   dmap_add_int(evbuf, "mstt", 200);    /* 12 */
   dmap_add_int(evbuf, "mtco", qp.results); /* 12 */
   dmap_add_int(evbuf, "mrco", nitems); /* 12 */
 
-  dmap_add_container(evbuf, tag, EVBUFFER_LENGTH(itemlist)); /* 8 */
+  dmap_add_container(evbuf, tag, len); /* 8 */
 
   db_query_end(&qp);
 
@@ -2215,7 +2321,8 @@ daap_reply_browse(struct evhttp_request *req, struct evbuffer *evbuf, char **uri
 
   if (sort_headers)
     {
-      dmap_add_container(evbuf, "mshl", EVBUFFER_LENGTH(sctx->headerlist)); /* 8 */
+      len = evbuffer_get_length(sctx->headerlist);
+      dmap_add_container(evbuf, "mshl", len); /* 8 */
       ret = evbuffer_add_buffer(evbuf, sctx->headerlist);
 
       if (ret < 0)
@@ -2255,6 +2362,7 @@ daap_reply_extra_data(struct evhttp_request *req, struct evbuffer *evbuf, char *
   struct evkeyvalq *headers;
   const char *param;
   char *ctype;
+  size_t len;
   int id;
   int max_w;
   int max_h;
@@ -2302,9 +2410,11 @@ daap_reply_extra_data(struct evhttp_request *req, struct evbuffer *evbuf, char *
     }
 
   if (strcmp(uri[2], "groups") == 0)
-    ret = artwork_get_group(id, max_w, max_h, evbuf);
+    ret = artwork_get_group(evbuf, id, max_w, max_h);
   else if (strcmp(uri[2], "items") == 0)
-    ret = artwork_get_item(id, max_w, max_h, evbuf);
+    ret = artwork_get_item(evbuf, id, max_w, max_h);
+
+  len = evbuffer_get_length(evbuf);
 
   switch (ret)
     {
@@ -2317,8 +2427,8 @@ daap_reply_extra_data(struct evhttp_request *req, struct evbuffer *evbuf, char *
 	break;
 
       default:
-	if (EVBUFFER_LENGTH(evbuf) > 0)
-	  evbuffer_drain(evbuf, EVBUFFER_LENGTH(evbuf));
+	if (len > 0)
+	  evbuffer_drain(evbuf, len);
 
 	goto no_artwork;
     }
@@ -2326,7 +2436,7 @@ daap_reply_extra_data(struct evhttp_request *req, struct evbuffer *evbuf, char *
   headers = evhttp_request_get_output_headers(req);
   evhttp_remove_header(headers, "Content-Type");
   evhttp_add_header(headers, "Content-Type", ctype);
-  snprintf(clen, sizeof(clen), "%ld", (long)EVBUFFER_LENGTH(evbuf));
+  snprintf(clen, sizeof(clen), "%ld", (long)len);
   evhttp_add_header(headers, "Content-Length", clen);
 
   /* No gzip compression for artwork */
@@ -2476,7 +2586,7 @@ daap_reply_dmap_test(struct evhttp_request *req, struct evbuffer *evbuf, char **
   dmap_add_field(test, &dmap_TST8, buf, 0);
   dmap_add_field(test, &dmap_TST9, buf, 0);
 
-  dmap_add_container(evbuf, dmap_TEST.tag, EVBUFFER_LENGTH(test));
+  dmap_add_container(evbuf, dmap_TEST.tag, evbuffer_get_length(test));
 
   ret = evbuffer_add_buffer(evbuf, test);
   evbuffer_free(test);
@@ -2889,7 +2999,7 @@ daap_init(void)
   int i;
   int ret;
 
-  next_session_id = DAAP_BASE_SESSION_ID;
+  srand((unsigned)time(NULL));
   current_rev = 2;
   update_requests = NULL;
 
@@ -2905,26 +3015,13 @@ daap_init(void)
         }
     }
 
-  daap_sessions = avl_alloc_tree(daap_session_compare, daap_session_free);
-  if (!daap_sessions)
-    {
-      DPRINTF(E_FATAL, L_DAAP, "DAAP init could not allocate DAAP sessions AVL tree\n");
-
-      goto daap_avl_alloc_fail;
-    }
-
   return 0;
-
- daap_avl_alloc_fail:
-  for (i = 0; daap_handlers[i].handler; i++)
-    regfree(&daap_handlers[i].preg);
-
-  return -1;
 }
 
 void
 daap_deinit(void)
 {
+  struct daap_session *s;
   struct daap_update_request *ur;
   struct evhttp_connection *evcon;
   int i;
@@ -2932,7 +3029,11 @@ daap_deinit(void)
   for (i = 0; daap_handlers[i].handler; i++)
     regfree(&daap_handlers[i].preg);
 
-  avl_free_tree(daap_sessions);
+  for (s = daap_sessions; daap_sessions; s = daap_sessions)
+    {
+      daap_sessions = s->next;
+      daap_session_free(s);
+    }
 
   for (ur = update_requests; update_requests; ur = update_requests)
     {
