@@ -37,6 +37,10 @@
 # include <sys/eventfd.h>
 #endif
 
+#include <event2/event.h>
+#include <event2/buffer.h>
+#include <event2/keyvalq_struct.h>
+
 #include "logger.h"
 #include "misc.h"
 #include "conffile.h"
@@ -45,7 +49,10 @@
 #include "httpd_dacp.h"
 #include "dmap_common.h"
 #include "db.h"
+#include "daap_query.h"
 #include "player.h"
+#include "queue.h"
+#include "listener.h"
 
 /* httpd event base, from httpd.c */
 extern struct event_base *evbase_httpd;
@@ -139,14 +146,14 @@ static int update_efd;
 #else
 static int update_pipe[2];
 #endif
-static struct event updateev;
+static struct event *updateev;
 static int current_rev;
 
 /* Play status update requests */
 static struct dacp_update_request *update_requests;
 
 /* Seek timer */
-static struct event seek_timer;
+static struct event *seek_timer;
 static int seek_target;
 
 
@@ -154,20 +161,40 @@ static int seek_target;
 static void
 dacp_nowplaying(struct evbuffer *evbuf, struct player_status *status, struct media_file_info *mfi)
 {
+  uint32_t id;
+  int64_t songalbumid;
+
   if ((status->status == PLAY_STOPPED) || !mfi)
     return;
 
+  /* Send bogus id's if playing internet radio, because clients like
+   * Remote and Retune will only update metadata (like artwork) if the id's
+   * change (which they wouldn't do if we sent the real ones)
+   * FIXME: Giving the client invalid ids on purpose is hardly ideal, but the
+   * clients don't seem to use these ids for anything other than rating.
+   */
+  if (mfi->data_kind == DATA_KIND_HTTP)
+    {
+      id = djb_hash(mfi->album, strlen(mfi->album));
+      songalbumid = (int64_t)id;
+    }
+  else
+    {
+      id = status->id;
+      songalbumid = mfi->songalbumid;
+    }
+
   dmap_add_container(evbuf, "canp", 16);
   dmap_add_raw_uint32(evbuf, 1); /* Database */
   dmap_add_raw_uint32(evbuf, status->plid);
   dmap_add_raw_uint32(evbuf, status->pos_pl);
-  dmap_add_raw_uint32(evbuf, status->id);
+  dmap_add_raw_uint32(evbuf, id);
 
   dmap_add_string(evbuf, "cann", mfi->title);
   dmap_add_string(evbuf, "cana", mfi->artist);
   dmap_add_string(evbuf, "canl", mfi->album);
   dmap_add_string(evbuf, "cang", mfi->genre);
-  dmap_add_long(evbuf, "asai", mfi->songalbumid);
+  dmap_add_long(evbuf, "asai", songalbumid);
 
   dmap_add_int(evbuf, "cmmk", 1);
 }
@@ -178,7 +205,11 @@ dacp_playingtime(struct evbuffer *evbuf, struct player_status *status, struct me
   if ((status->status == PLAY_STOPPED) || !mfi)
     return;
 
-  dmap_add_int(evbuf, "cant", mfi->song_length - status->pos_ms); /* Remaining time in ms */
+  if (mfi->song_length)
+    dmap_add_int(evbuf, "cant", mfi->song_length - status->pos_ms); /* Remaining time in ms */
+  else
+    dmap_add_int(evbuf, "cant", 0); /* Unknown remaining time */
+
   dmap_add_int(evbuf, "cast", mfi->song_length); /* Song length in ms */
 }
 
@@ -247,7 +278,7 @@ make_playstatusupdate(struct evbuffer *evbuf)
   dmap_add_char(psu, "casu", 1);              /*  9 */ /* unknown */
   dmap_add_char(psu, "ceQu", 0);              /*  9 */ /* unknown */
 
-  dmap_add_container(evbuf, "cmst", EVBUFFER_LENGTH(psu));    /* 8 + len */
+  dmap_add_container(evbuf, "cmst", evbuffer_get_length(psu));    /* 8 + len */
 
   ret = evbuffer_add_buffer(evbuf, psu);
   evbuffer_free(psu);
@@ -268,6 +299,8 @@ playstatusupdate_cb(int fd, short what, void *arg)
   struct evbuffer *evbuf;
   struct evbuffer *update;
   struct evhttp_connection *evcon;
+  uint8_t *buf;
+  size_t len;
   int ret;
 
 #ifdef USE_EVENTFD
@@ -309,6 +342,8 @@ playstatusupdate_cb(int fd, short what, void *arg)
   if (ret < 0)
     goto out_free_update;
 
+  len = evbuffer_get_length(update);
+
   for (ur = update_requests; update_requests; ur = update_requests)
     {
       update_requests = ur->next;
@@ -317,9 +352,15 @@ playstatusupdate_cb(int fd, short what, void *arg)
       if (evcon)
 	evhttp_connection_set_closecb(evcon, NULL, NULL);
 
-      evbuffer_add(evbuf, EVBUFFER_DATA(update), EVBUFFER_LENGTH(update));
-
-      httpd_send_reply(ur->req, HTTP_OK, "OK", evbuf);
+      // Only copy buffer if we actually need to reuse it
+      if (ur->next)
+	{
+	  buf = evbuffer_pullup(update, -1);
+	  evbuffer_add(evbuf, buf, len);
+	  httpd_send_reply(ur->req, HTTP_OK, "OK", evbuf);
+	}
+      else
+	httpd_send_reply(ur->req, HTTP_OK, "OK", update);
 
       free(ur);
     }
@@ -331,17 +372,21 @@ playstatusupdate_cb(int fd, short what, void *arg)
  out_free_evbuf:
   evbuffer_free(evbuf);
  readd:
-  ret = event_add(&updateev, NULL);
+  ret = event_add(updateev, NULL);
   if (ret < 0)
     DPRINTF(E_LOG, L_DACP, "Couldn't re-add event for playstatusupdate\n");
 }
 
 /* Thread: player */
 static void
-dacp_playstatus_update_handler(void)
+dacp_playstatus_update_handler(enum listener_event_type type)
 {
   int ret;
 
+  // Only send status update on player change events
+  if (type != LISTENER_PLAYER)
+    return;
+
 #ifdef USE_EVENTFD
   ret = eventfd_write(update_efd, 1);
   if (ret < 0)
@@ -569,9 +614,6 @@ dacp_propset_playingtime(const char *value, struct evkeyvalq *query)
   struct timeval tv;
   int ret;
 
-  if (event_initialized(&seek_timer))
-    event_del(&seek_timer);
-
   ret = safe_atoi32(value, &seek_target);
   if (ret < 0)
     {
@@ -580,11 +622,9 @@ dacp_propset_playingtime(const char *value, struct evkeyvalq *query)
       return;
     }
 
-  evtimer_set(&seek_timer, seek_timer_cb, NULL);
-  event_base_set(evbase_httpd, &seek_timer);
   evutil_timerclear(&tv);
   tv.tv_usec = 200 * 1000;
-  evtimer_add(&seek_timer, &tv);
+  evtimer_add(seek_timer, &tv);
 }
 
 static void
@@ -667,11 +707,21 @@ dacp_propset_userrating(const char *value, struct evkeyvalq *query)
     }
 
   mfi = db_file_fetch_byid(itemid);
+
+  /* If no mfi, it may be because we sent an invalid nowplaying itemid. In this
+   * case request the real one from the player and default to that.
+   */
   if (!mfi)
     {
-      DPRINTF(E_LOG, L_DACP, "Could not fetch file id %d\n", itemid);
+      DPRINTF(E_WARN, L_DACP, "Invalid id %d for rating, defaulting to player id\n", itemid);
 
-      return;
+      ret = player_now_playing(&itemid);
+      if ((ret < 0) || !(mfi = db_file_fetch_byid(itemid)))
+	{
+	  DPRINTF(E_WARN, L_DACP, "Could not find an id for rating\n");
+
+	  return;
+	}
     }
 
   mfi->rating = rating;
@@ -716,16 +766,223 @@ dacp_reply_ctrlint(struct evhttp_request *req, struct evbuffer *evbuf, char **ur
   httpd_send_reply(req, HTTP_OK, "OK", evbuf);
 }
 
+static int
+find_first_song_id(const char *query)
+{
+  struct db_media_file_info dbmfi;
+  struct query_params qp;
+  int id;
+  int ret;
+
+  memset(&qp, 0, sizeof(struct query_params));
+
+  /* We only want the id of the first song */
+  qp.type = Q_ITEMS;
+  qp.idx_type = I_FIRST;
+  qp.sort = S_NONE;
+  qp.offset = 0;
+  qp.limit = 1;
+  qp.filter = daap_query_parse_sql(query);
+  if (!qp.filter)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Improper DAAP query!\n");
+
+      return -1;
+    }
+
+  ret = db_query_start(&qp);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_PLAYER, "Could not start query\n");
+
+      goto no_query_start;
+    }
+
+  if (((ret = db_query_fetch_file(&qp, &dbmfi)) == 0) && (dbmfi.id))
+    {
+      ret = safe_atoi32(dbmfi.id, &id);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_PLAYER, "Invalid song id in query result!\n");
+
+	  goto no_result;
+	}
+
+      DPRINTF(E_DBG, L_PLAYER, "Found index song (id %d)\n", id);
+      ret = 1;
+    }
+  else
+    {
+      DPRINTF(E_LOG, L_PLAYER, "No song matches query (results %d): %s\n", qp.results, qp.filter);
+
+      goto no_result;
+    }
+
+ no_result:
+  db_query_end(&qp);
+
+ no_query_start:
+  if (qp.filter)
+    free(qp.filter);
+
+  if (ret == 1)
+    return id;
+  else
+    return -1;
+}
+
+
+static int
+dacp_queueitem_make(struct queue_item **head, const char *query, const char *queuefilter, const char *sort, int quirk)
+{
+  struct media_file_info *mfi;
+  struct query_params qp;
+  struct queue_item *items;
+  int64_t albumid;
+  int64_t artistid;
+  int plid;
+  int id;
+  int idx;
+  int ret;
+  int len;
+  char *s;
+  char buf[1024];
+
+  if (query)
+    {
+      id = find_first_song_id(query);
+      if (id < 0)
+        return -1;
+    }
+  else
+    id = 0;
+
+  memset(&qp, 0, sizeof(struct query_params));
+
+  qp.offset = 0;
+  qp.limit = 0;
+  qp.sort = S_NONE;
+  qp.idx_type = I_NONE;
+
+  if (quirk)
+    {
+      qp.sort = S_ALBUM;
+      qp.type = Q_ITEMS;
+      mfi = db_file_fetch_byid(id);
+      if (!mfi)
+	return -1;
+      snprintf(buf, sizeof(buf), "f.songalbumid = %" PRIi64, mfi->songalbumid);
+      free_mfi(mfi, 0);
+      qp.filter = strdup(buf);
+    }
+  else if (queuefilter)
+    {
+      len = strlen(queuefilter);
+      if ((len > 6) && (strncmp(queuefilter, "album:", 6) == 0))
+	{
+	  qp.type = Q_ITEMS;
+	  ret = safe_atoi64(strchr(queuefilter, ':') + 1, &albumid);
+	  if (ret < 0)
+	    {
+	      DPRINTF(E_LOG, L_PLAYER, "Invalid album id in queuefilter: %s\n", queuefilter);
+
+	      return -1;
+	    }
+	  snprintf(buf, sizeof(buf), "f.songalbumid = %" PRIi64, albumid);
+	  qp.filter = strdup(buf);
+	}
+      else if ((len > 7) && (strncmp(queuefilter, "artist:", 7) == 0))
+	{
+	  qp.type = Q_ITEMS;
+	  ret = safe_atoi64(strchr(queuefilter, ':') + 1, &artistid);
+	  if (ret < 0)
+	    {
+	      DPRINTF(E_LOG, L_PLAYER, "Invalid artist id in queuefilter: %s\n", queuefilter);
+
+	      return -1;
+	    }
+	  snprintf(buf, sizeof(buf), "f.songartistid = %" PRIi64, artistid);
+	  qp.filter = strdup(buf);
+	}
+      else if ((len > 9) && (strncmp(queuefilter, "playlist:", 9) == 0))
+	{
+	  qp.type = Q_PLITEMS;
+	  ret = safe_atoi32(strchr(queuefilter, ':') + 1, &plid);
+	  if (ret < 0)
+	    {
+	      DPRINTF(E_LOG, L_PLAYER, "Invalid playlist id in queuefilter: %s\n", queuefilter);
+
+	      return -1;
+	    }
+	  qp.id = plid;
+	  qp.filter = strdup("1 = 1");
+	}
+      else if ((len > 6) && (strncmp(queuefilter, "genre:", 6) == 0))
+	{
+	  qp.type = Q_ITEMS;
+	  s = db_escape_string(queuefilter + 6);
+	  if (!s)
+	    return -1;
+	  snprintf(buf, sizeof(buf), "f.genre = '%s'", s);
+	  qp.filter = strdup(buf);
+	}
+      else
+	{
+	  DPRINTF(E_LOG, L_PLAYER, "Unknown queuefilter %s\n", queuefilter);
+
+	  // If the queuefilter is unkown, ignore it and use the query parameter instead to build the sql query
+	  id = 0;
+	  qp.type = Q_ITEMS;
+	  qp.filter = daap_query_parse_sql(query);
+	}
+    }
+  else
+    {
+      id = 0;
+      qp.type = Q_ITEMS;
+      qp.filter = daap_query_parse_sql(query);
+    }
+
+  if (sort)
+    {
+      if (strcmp(sort, "name") == 0)
+	qp.sort = S_NAME;
+      else if (strcmp(sort, "album") == 0)
+	qp.sort = S_ALBUM;
+      else if (strcmp(sort, "artist") == 0)
+	qp.sort = S_ARTIST;
+    }
+
+  items = queueitem_make_byquery(&qp);
+
+  if (qp.filter)
+    free(qp.filter);
+
+  if (items)
+    *head = items;
+  else
+    return -1;
+
+  // Get the position (0-based) of the first item
+  idx = queueitem_pos(items, id);
+
+  return idx;
+}
+
 static void
 dacp_reply_cue_play(struct evhttp_request *req, struct evbuffer *evbuf, char **uri, struct evkeyvalq *query)
 {
   struct player_status status;
-  struct player_source *ps;
+  struct queue_item *items;
   const char *sort;
   const char *cuequery;
   const char *param;
   uint32_t id;
+  uint32_t item_id;
+  uint32_t pos;
   int clear;
+  struct player_history *history;
+  int hist;
   int ret;
 
   /* /cue?command=play&query=...&sort=...&index=N */
@@ -749,7 +1006,7 @@ dacp_reply_cue_play(struct evhttp_request *req, struct evbuffer *evbuf, char **u
     {
       sort = evhttp_find_header(query, "sort");
 
-      ret = player_queue_make_daap(&ps, cuequery, NULL, sort, 0);
+      ret = dacp_queueitem_make(&items, cuequery, NULL, sort, 0);
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_DACP, "Could not build song queue\n");
@@ -758,7 +1015,7 @@ dacp_reply_cue_play(struct evhttp_request *req, struct evbuffer *evbuf, char **u
 	  return;
 	}
 
-      player_queue_add(ps);
+      player_queue_add(items);
     }
   else
     {
@@ -773,19 +1030,53 @@ dacp_reply_cue_play(struct evhttp_request *req, struct evbuffer *evbuf, char **u
     dacp_propset_shufflestate(param, NULL);
 
   id = 0;
+  item_id = 0;
+  pos = 0;
   param = evhttp_find_header(query, "index");
   if (param)
     {
-      ret = safe_atou32(param, &id);
+      ret = safe_atou32(param, &pos);
       if (ret < 0)
 	DPRINTF(E_LOG, L_DACP, "Invalid index (%s) in cue request\n", param);
     }
 
-  /* If selection was from Up Next queue (command will be playnow), then index is relative */
+  /* If selection was from Up Next queue or history queue (command will be playnow), then index is relative */
+  hist = 0;
   if ((param = evhttp_find_header(query, "command")) && (strcmp(param, "playnow") == 0))
-    id += status.pos_pl;
+    {
+      /* If mode parameter is -1 or 1, the index is relative to the history queue, otherwise to the Up Next queue */
+      param = evhttp_find_header(query, "mode");
+      if (param && ((strcmp(param, "-1") == 0) || (strcmp(param, "1") == 0)))
+	{
+	  /* Play from history queue */
+	  hist = 1;
+	  history = player_history_get();
+	  if (history->count > pos)
+	    {
+	      pos = (history->start_index + history->count - pos - 1) % MAX_HISTORY_COUNT;
+	      item_id = history->item_id[pos];
+	    }
+	  else
+	    {
+	      DPRINTF(E_LOG, L_DACP, "Could not start playback from history\n");
+
+	      dmap_send_error(req, "cacr", "Playback failed to start");
+	      return;
+	    }
+	}
+      else
+	{
+	  /* Play from Up Next queue */
+	  pos += status.pos_pl;
+	}
+    }
+
+  /* If playing from history queue, the pos holds the id of the item to play */
+  if (hist)
+    ret = player_playback_start_byitemid(item_id, &id);
+  else
+    ret = player_playback_start_bypos(pos, &id);
 
-  ret = player_playback_start(&id);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_DACP, "Could not start playback\n");
@@ -853,12 +1144,13 @@ static void
 dacp_reply_playspec(struct evhttp_request *req, struct evbuffer *evbuf, char **uri, struct evkeyvalq *query)
 {
   struct player_status status;
-  struct player_source *ps;
+  struct queue_item *items;
   struct daap_session *s;
   const char *param;
   const char *shuffle;
   uint32_t plid;
   uint32_t id;
+  int pos;
   int ret;
 
   /* /ctrl-int/1/playspec?database-spec='dmap.persistentid:0x1'&container-spec='dmap.persistentid:0x5'&container-item-spec='dmap.containeritemid:0x9'
@@ -934,15 +1226,26 @@ dacp_reply_playspec(struct evhttp_request *req, struct evbuffer *evbuf, char **u
 
   DPRINTF(E_DBG, L_DACP, "Playspec request for playlist %d, start song id %d%s\n", plid, id, (shuffle) ? ", shuffle" : "");
 
-  ps = player_queue_make_pl(plid, &id);
-  if (!ps)
+  items = NULL;
+  if (plid > 0)
+    items = queueitem_make_byplid(plid);
+  else if (id > 0)
+    items = queueitem_make_byid(id);
+
+  if (!items)
     {
       DPRINTF(E_LOG, L_DACP, "Could not build song queue from playlist %d\n", plid);
 
       goto out_fail;
     }
 
-  DPRINTF(E_DBG, L_DACP, "Playspec start song index is %d\n", id);
+  pos = queueitem_pos(items, id);
+  if (pos < 0)
+    {
+      DPRINTF(E_DBG, L_DACP, "No item with %d found in queue\n", id);
+      pos = 0;
+    }
+  DPRINTF(E_DBG, L_DACP, "Playspec start song index is %d\n", pos);
 
   player_get_status(&status);
 
@@ -950,13 +1253,13 @@ dacp_reply_playspec(struct evhttp_request *req, struct evbuffer *evbuf, char **u
     player_playback_stop();
 
   player_queue_clear();
-  player_queue_add(ps);
+  player_queue_add(items);
   player_queue_plid(plid);
 
   if (shuffle)
     dacp_propset_shufflestate(shuffle, NULL);
 
-  ret = player_playback_start(&id);
+  ret = player_playback_start_bypos(pos, NULL);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_DACP, "Could not start playback\n");
@@ -1129,15 +1432,6 @@ dacp_reply_playresume(struct evhttp_request *req, struct evbuffer *evbuf, char *
   evhttp_send_reply(req, HTTP_NOCONTENT, "No Content", evbuf);
 }
 
-static struct player_source *
-next_ps(struct player_source *ps, char shuffle)
-{
-  if (shuffle)
-    return ps->shuffle_next;
-  else
-    return ps->pl_next;
-}
-
 static int
 playqueuecontents_add_source(struct evbuffer *songlist, uint32_t source_id, int pos_in_queue, uint32_t plid)
 {
@@ -1175,7 +1469,7 @@ playqueuecontents_add_source(struct evbuffer *songlist, uint32_t source_id, int
   dmap_add_char(song, "caks", 6); /* Unknown */
   dmap_add_int(song, "ceQI", pos_in_queue);
 
-  dmap_add_container(songlist, "mlit", EVBUFFER_LENGTH(song));
+  dmap_add_container(songlist, "mlit", evbuffer_get_length(song));
 
   ret = evbuffer_add_buffer(songlist, song);
   evbuffer_free(song);
@@ -1197,15 +1491,17 @@ dacp_reply_playqueuecontents(struct evhttp_request *req, struct evbuffer *evbuf,
   struct daap_session *s;
   struct evbuffer *songlist;
   struct evbuffer *playlists;
-  struct player_source *ps;
-  struct player_source *head;
   struct player_status status;
   struct player_history *history;
+  struct queue *queue;
+  struct queue_item *item;
   const char *param;
+  size_t songlist_length;
+  size_t playlist_length;
   int span;
+  int count;
   int i;
   int n;
-  int songlist_length;
   int ret;
   int start_index;
 
@@ -1271,19 +1567,15 @@ dacp_reply_playqueuecontents(struct evhttp_request *req, struct evbuffer *evbuf,
       /* Get queue and make songlist only if playing or paused */
       if (status.status != PLAY_STOPPED)
 	{
-	  /* Fast forward to song currently being played */
-	  head = player_queue_get();
-	  if (head)
+	  queue = player_queue_get_bypos(abs(span));
+	  if (queue)
 	    {
-	      ps = head;
-	      while ((ps->id != status.id) && (ps = next_ps(ps, status.shuffle)) && (ps != head))
-		i++;
-
-	      while ((n < abs(span)) && (ps = next_ps(ps, status.shuffle)) && (ps != head))
+	      i = 0;
+	      count = queue_count(queue);
+	      for (n = 0; (n < count) && (n < abs(span)); n++)
 		{
-		  n++;
-
-		  ret = playqueuecontents_add_source(songlist, ps->id, (n + i + 1), status.plid);
+		  item = queue_get_byindex(queue, n, 0);
+		  ret = playqueuecontents_add_source(songlist, queueitem_id(item), (n + i + 1), status.plid);
 		  if (ret < 0)
 		    {
 		      DPRINTF(E_LOG, L_DACP, "Could not add song to songlist for playqueue-contents\n");
@@ -1292,6 +1584,7 @@ dacp_reply_playqueuecontents(struct evhttp_request *req, struct evbuffer *evbuf,
 		      return;
 		    }
 		}
+	      queue_free(queue);
 	    }
 	}
     }
@@ -1328,19 +1621,20 @@ dacp_reply_playqueuecontents(struct evhttp_request *req, struct evbuffer *evbuf,
       dmap_add_string(playlists, "ceQl", "Up Next");         /* 15 = 8 + 7 */
       dmap_add_string(playlists, "ceQh", "from Music");      /* 18 = 8 + 10 */
 
-      songlist_length = EVBUFFER_LENGTH(songlist);
+      songlist_length = evbuffer_get_length(songlist);
     }
   else
     songlist_length = 0;
 
   /* Final construction of reply */
-  dmap_add_container(evbuf, "ceQR", 79 + EVBUFFER_LENGTH(playlists) + songlist_length); /* size of entire container */
+  playlist_length = evbuffer_get_length(playlists);
+  dmap_add_container(evbuf, "ceQR", 79 + playlist_length + songlist_length); /* size of entire container */
   dmap_add_int(evbuf, "mstt", 200);                                                     /* 12, dmap.status */
   dmap_add_int(evbuf, "mtco", abs(span));                                               /* 12 */
   dmap_add_int(evbuf, "mrco", n);                                                       /* 12 */
   dmap_add_char(evbuf, "ceQu", 0);                                                      /*  9 */
-  dmap_add_container(evbuf, "mlcl", 8 + EVBUFFER_LENGTH(playlists) + songlist_length);  /*  8 */
-  dmap_add_container(evbuf, "ceQS", EVBUFFER_LENGTH(playlists));                        /*  8 */
+  dmap_add_container(evbuf, "mlcl", 8 + playlist_length + songlist_length);  /*  8 */
+  dmap_add_container(evbuf, "ceQS", playlist_length);                        /*  8 */
   ret = evbuffer_add_buffer(evbuf, playlists);
   evbuffer_free(playlists);
   if (ret < 0)
@@ -1375,9 +1669,7 @@ static void
 dacp_reply_playqueueedit_clear(struct evhttp_request *req, struct evbuffer *evbuf, char **uri, struct evkeyvalq *query)
 {
   const char *param;
-  int clear_hist;
 
-  clear_hist = 0;
   param = evhttp_find_header(query, "mode");
 
   /*
@@ -1386,9 +1678,9 @@ dacp_reply_playqueueedit_clear(struct evhttp_request *req, struct evbuffer *evbu
    * otherwise the current playlist.
    */
   if (strcmp(param,"0x68697374") == 0)
-    clear_hist = 1;
-
-  player_queue_empty(clear_hist);
+    player_queue_clear_history();
+  else
+    player_queue_clear();
 
   dmap_add_container(evbuf, "cacr", 24); /* 8 + len */
   dmap_add_int(evbuf, "mstt", 200);      /* 12 */
@@ -1409,7 +1701,7 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
   //?command=add&query='dmap.itemid:2'&query-modifier=containers&sort=name&mode=2&session-id=100
   // -> mode 2: stop playblack, clear playqueue, add shuffled songs from playlist=itemid to playqueue
 
-  struct player_source *ps;
+  struct queue_item *items;
   const char *editquery;
   const char *queuefilter;
   const char *querymodifier;
@@ -1461,7 +1753,7 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
       if (!querymodifier || (strcmp(querymodifier, "containers") != 0))
 	{
 	  quirkyquery = (mode == 1) && strstr(editquery, "dmap.itemid:") && ((!queuefilter) || strstr(queuefilter, "(null)"));
-	  ret = player_queue_make_daap(&ps, editquery, queuefilter, sort, quirkyquery);
+	  ret = dacp_queueitem_make(&items, editquery, queuefilter, sort, quirkyquery);
 	}
       else
 	{
@@ -1476,7 +1768,7 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
 	    }
 	  
 	  snprintf(modifiedquery, sizeof(modifiedquery), "playlist:%d", plid);
-	  ret = player_queue_make_daap(&ps, NULL, modifiedquery, sort, 0);
+	  ret = dacp_queueitem_make(&items, NULL, modifiedquery, sort, 0);
 	}
 
       if (ret < 0)
@@ -1491,11 +1783,11 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
 
       if (mode == 3)
       {
-        player_queue_add_next(ps);
+        player_queue_add_next(items);
       }
       else
       {
-        player_queue_add(ps);
+        player_queue_add(items);
       }
     }
   else
@@ -1513,7 +1805,7 @@ dacp_reply_playqueueedit_add(struct evhttp_request *req, struct evbuffer *evbuf,
     }
 
   DPRINTF(E_DBG, L_DACP, "Song queue built, playback starting at index %" PRIu32 "\n", idx);
-  ret = player_playback_start(&idx);
+  ret = player_playback_start_bypos(idx, NULL);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_DACP, "Could not start playback\n");
@@ -1564,7 +1856,7 @@ dacp_reply_playqueueedit_move(struct evhttp_request *req, struct evbuffer *evbuf
       return;
     }
 
-    player_queue_move(src, dst);
+    player_queue_move_bypos(src, dst);
   }
 
   /* 204 No Content is the canonical reply */
@@ -1596,7 +1888,7 @@ dacp_reply_playqueueedit_remove(struct evhttp_request *req, struct evbuffer *evb
       return;
     }
 
-    player_queue_remove(item_index);
+    player_queue_remove_bypos(item_index);
   }
 
   /* 204 No Content is the canonical reply */
@@ -1758,6 +2050,7 @@ dacp_reply_nowplayingartwork(struct evhttp_request *req, struct evbuffer *evbuf,
   struct evkeyvalq *headers;
   const char *param;
   char *ctype;
+  size_t len;
   uint32_t id;
   int max_w;
   int max_h;
@@ -1807,7 +2100,9 @@ dacp_reply_nowplayingartwork(struct evhttp_request *req, struct evbuffer *evbuf,
   if (ret < 0)
     goto no_artwork;
 
-  ret = artwork_get_item(id, max_w, max_h, evbuf);
+  ret = artwork_get_item(evbuf, id, max_w, max_h);
+  len = evbuffer_get_length(evbuf);
+
   switch (ret)
     {
       case ART_FMT_PNG:
@@ -1819,8 +2114,8 @@ dacp_reply_nowplayingartwork(struct evhttp_request *req, struct evbuffer *evbuf,
 	break;
 
       default:
-	if (EVBUFFER_LENGTH(evbuf) > 0)
-	  evbuffer_drain(evbuf, EVBUFFER_LENGTH(evbuf));
+	if (len > 0)
+	  evbuffer_drain(evbuf, len);
 
 	goto no_artwork;
     }
@@ -1828,7 +2123,7 @@ dacp_reply_nowplayingartwork(struct evhttp_request *req, struct evbuffer *evbuf,
   headers = evhttp_request_get_output_headers(req);
   evhttp_remove_header(headers, "Content-Type");
   evhttp_add_header(headers, "Content-Type", ctype);
-  snprintf(clen, sizeof(clen), "%ld", (long)EVBUFFER_LENGTH(evbuf));
+  snprintf(clen, sizeof(clen), "%ld", (long)len);
   evhttp_add_header(headers, "Content-Length", clen);
 
   /* No gzip compression for artwork */
@@ -1851,6 +2146,7 @@ dacp_reply_getproperty(struct evhttp_request *req, struct evbuffer *evbuf, char
   char *ptr;
   char *prop;
   char *propstr;
+  size_t len;
   int ret;
 
   s = daap_session_find(req, query, evbuf);
@@ -1922,7 +2218,8 @@ dacp_reply_getproperty(struct evhttp_request *req, struct evbuffer *evbuf, char
   if (mfi)
     free_mfi(mfi, 0);
 
-  dmap_add_container(evbuf, "cmgt", 12 + EVBUFFER_LENGTH(proplist)); /* 8 + len */
+  len = evbuffer_get_length(proplist);
+  dmap_add_container(evbuf, "cmgt", 12 + len);
   dmap_add_int(evbuf, "mstt", 200);      /* 12 */
 
   ret = evbuffer_add_buffer(evbuf, proplist);
@@ -2020,6 +2317,7 @@ dacp_reply_getspeakers(struct evhttp_request *req, struct evbuffer *evbuf, char
 {
   struct daap_session *s;
   struct evbuffer *spklist;
+  size_t len;
 
   s = daap_session_find(req, query, evbuf);
   if (!s)
@@ -2037,7 +2335,8 @@ dacp_reply_getspeakers(struct evhttp_request *req, struct evbuffer *evbuf, char
 
   player_speaker_enumerate(speaker_enum_cb, spklist);
 
-  dmap_add_container(evbuf, "casp", 12 + EVBUFFER_LENGTH(spklist)); /* 8 + len */
+  len = evbuffer_get_length(spklist);
+  dmap_add_container(evbuf, "casp", 12 + len);
   dmap_add_int(evbuf, "mstt", 200); /* 12 */
 
   evbuffer_add_buffer(evbuf, spklist);
@@ -2364,11 +2663,7 @@ dacp_init(void)
       return -1;
     }
 #else
-# if defined(__linux__)
   ret = pipe2(update_pipe, O_CLOEXEC);
-# else
-  ret = pipe(update_pipe);
-# endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_DACP, "Could not create update pipe: %s\n", strerror(errno));
@@ -2390,14 +2685,27 @@ dacp_init(void)
     }
 
 #ifdef USE_EVENTFD
-  event_set(&updateev, update_efd, EV_READ, playstatusupdate_cb, NULL);
+  updateev = event_new(evbase_httpd, update_efd, EV_READ, playstatusupdate_cb, NULL);
 #else
-  event_set(&updateev, update_pipe[0], EV_READ, playstatusupdate_cb, NULL);
+  updateev = event_new(evbase_httpd, update_pipe[0], EV_READ, playstatusupdate_cb, NULL);
 #endif
-  event_base_set(evbase_httpd, &updateev);
-  event_add(&updateev, NULL);
+  if (!updateev)
+    {
+      DPRINTF(E_LOG, L_DACP, "Could not create update event\n");
 
-  player_set_update_handler(dacp_playstatus_update_handler);
+      return -1;
+    }
+  event_add(updateev, NULL);
+
+  seek_timer = evtimer_new(evbase_httpd, seek_timer_cb, NULL);
+  if (!seek_timer)
+    {
+      DPRINTF(E_LOG, L_DACP, "Could not create seek_timer event\n");
+
+      return -1;
+    }
+
+  listener_add(dacp_playstatus_update_handler, LISTENER_PLAYER);
 
   return 0;
 
@@ -2418,7 +2726,9 @@ dacp_deinit(void)
   struct evhttp_connection *evcon;
   int i;
 
-  player_set_update_handler(NULL);
+  listener_remove(dacp_playstatus_update_handler);
+
+  event_free(seek_timer);
 
   for (i = 0; dacp_handlers[i].handler; i++)
     regfree(&dacp_handlers[i].preg);
@@ -2437,7 +2747,7 @@ dacp_deinit(void)
       free(ur);
     }
 
-  event_del(&updateev);
+  event_free(updateev);
 
 #ifdef USE_EVENTFD
   close(update_efd);
