@@ -57,11 +57,11 @@
 #include <arpa/inet.h>
 #include <net/if.h>
 
-#include <event.h>
-#include "evrtsp/evrtsp.h"
-
+#include <event2/event.h>
+#include <event2/buffer.h>
 #include <gcrypt.h>
 
+#include "evrtsp/evrtsp.h"
 #include "conffile.h"
 #include "logger.h"
 #include "misc.h"
@@ -109,6 +109,8 @@ struct raop_session
   unsigned auth_quirk_itunes:1;
   unsigned wants_metadata:1;
 
+  struct event *deferredev;
+
   int reqs_in_flight;
   int cseq;
   char *session;
@@ -161,13 +163,7 @@ struct raop_service
 {
   int fd;
   unsigned short port;
-  struct event ev;
-};
-
-struct raop_deferred_eh
-{
-  struct event ev;
-  struct raop_session *session;
+  struct event *ev;
 };
 
 typedef void (*evrtsp_req_cb)(struct evrtsp_request *req, void *arg);
@@ -242,7 +238,7 @@ static struct raop_metadata *metadata_head;
 static struct raop_metadata *metadata_tail;
 
 /* FLUSH timer */
-static struct event flush_timer;
+static struct event *flush_timer;
 
 /* Sessions */
 static struct raop_session *sessions;
@@ -751,8 +747,9 @@ raop_metadata_prune(uint64_t rtptime)
     }
 }
 
-static struct raop_metadata *
-raop_metadata_prepare(int id, uint64_t rtptime)
+/* Thread: worker */
+struct raop_metadata *
+raop_metadata_prepare(int id)
 {
   struct query_params qp;
   struct db_media_file_info dbmfi;
@@ -772,6 +769,28 @@ raop_metadata_prepare(int id, uint64_t rtptime)
 
   memset(rmd, 0, sizeof(struct raop_metadata));
 
+  /* Get artwork */
+  rmd->artwork = evbuffer_new();
+  if (!rmd->artwork)
+    {
+      DPRINTF(E_LOG, L_RAOP, "Out of memory for artwork evbuffer; no artwork will be sent\n");
+
+      goto skip_artwork;
+    }
+
+  ret = artwork_get_item(rmd->artwork, id, 600, 600);
+  if (ret < 0)
+    {
+      DPRINTF(E_INFO, L_RAOP, "Failed to retrieve artwork for file id %d; no artwork will be sent\n", id);
+
+      evbuffer_free(rmd->artwork);
+      rmd->artwork = NULL;
+    }
+
+  rmd->artwork_fmt = ret;
+
+ skip_artwork:
+
   /* Get dbmfi */
   memset(&qp, 0, sizeof(struct query_params));
   qp.type = Q_ITEMS;
@@ -839,40 +858,11 @@ raop_metadata_prepare(int id, uint64_t rtptime)
       goto out_metadata;
     }
 
-  rmd->start = rtptime;
-  rmd->end = rtptime + (duration * 44100UL) / 1000UL;
-
-  /* Get artwork */
-  rmd->artwork = evbuffer_new();
-  if (!rmd->artwork)
-    {
-      DPRINTF(E_LOG, L_RAOP, "Out of memory for artwork evbuffer; no artwork will be sent\n");
-
-      goto skip_artwork;
-    }
-
-  ret = artwork_get_item(id, 600, 600, rmd->artwork);
-  if (ret < 0)
-    {
-      DPRINTF(E_INFO, L_RAOP, "Failed to retrieve artwork for '%s' (%d); no artwork will be sent\n", dbmfi.title, id);
-
-      evbuffer_free(rmd->artwork);
-      rmd->artwork = NULL;
-    }
-
-  rmd->artwork_fmt = ret;
-
- skip_artwork:
   db_query_end(&qp);
 
-  /* Add rmd to metadata list */
-  if (metadata_tail)
-    metadata_tail->next = rmd;
-  else
-    {
-      metadata_head = rmd;
-      metadata_tail = rmd;
-    }
+  /* raop_metadata_send() will add rtptime to these */
+  rmd->start = 0;
+  rmd->end = (duration * 44100UL) / 1000UL;
 
   return rmd;
 
@@ -1645,6 +1635,8 @@ raop_session_free(struct raop_session *rs)
 
   evrtsp_connection_free(rs->ctrl);
 
+  event_free(rs->deferredev);
+
   close(rs->server_fd);
 
   if (rs->realm)
@@ -1714,16 +1706,11 @@ raop_session_failure(struct raop_session *rs)
 }
 
 static void
-raop_deferred_eh_cb(int fd, short what, void *arg)
+raop_deferredev_cb(int fd, short what, void *arg)
 {
-  struct raop_deferred_eh *deh;
   struct raop_session *rs;
 
-  deh = (struct raop_deferred_eh *)arg;
-
-  rs = deh->session;
-
-  free(deh);
+  rs = (struct raop_session *)arg;
 
   DPRINTF(E_DBG, L_RAOP, "Cleaning up failed session (deferred) on device %s\n", rs->devname);
 
@@ -1734,7 +1721,6 @@ static void
 raop_rtsp_close_cb(struct evrtsp_connection *evcon, void *arg)
 {
   struct timeval tv;
-  struct raop_deferred_eh *deh;
   struct raop_session *rs;
 
   rs = (struct raop_session *)arg;
@@ -1743,20 +1729,8 @@ raop_rtsp_close_cb(struct evrtsp_connection *evcon, void *arg)
 
   rs->state = RAOP_FAILED;
 
-  deh = (struct raop_deferred_eh *)malloc(sizeof(struct raop_deferred_eh));
-  if (!deh)
-    {
-      DPRINTF(E_LOG, L_RAOP, "Out of memory for deferred error handling!\n");
-
-      return;
-    }
-
-  deh->session = rs;
-
-  evtimer_set(&deh->ev, raop_deferred_eh_cb, deh);
-  event_base_set(evbase_player, &deh->ev);
   evutil_timerclear(&tv);
-  evtimer_add(&deh->ev, &tv);
+  evtimer_add(rs->deferredev, &tv);
 }
 
 static struct raop_session *
@@ -1839,12 +1813,20 @@ raop_session_make(struct raop_device *rd, int family, raop_status_cb cb)
 	break;
     }
 
+  rs->deferredev = evtimer_new(evbase_player, raop_deferredev_cb, rs);
+  if (!rs->deferredev)
+    {
+      DPRINTF(E_LOG, L_RAOP, "Out of memory for deferred error handling!\n");
+
+      goto out_free_rs;
+    }
+
   rs->ctrl = evrtsp_connection_new(address, port);
   if (!rs->ctrl)
     {
       DPRINTF(E_LOG, L_RAOP, "Could not create control connection to %s\n", address);
 
-      goto out_free_rs;
+      goto out_free_event;
     }
 
   evrtsp_connection_set_base(rs->ctrl, evbase_player);
@@ -1911,6 +1893,8 @@ raop_session_make(struct raop_device *rd, int family, raop_status_cb cb)
 
  out_free_evcon:
   evrtsp_connection_free(rs->ctrl);
+ out_free_event:
+  event_free(rs->deferredev);
  out_free_rs:
   free(rs);
 
@@ -2005,6 +1989,8 @@ static int
 raop_metadata_send_artwork(struct raop_session *rs, struct evbuffer *evbuf, struct raop_metadata *rmd, char *rtptime)
 {
   char *ctype;
+  uint8_t *buf;
+  size_t len;
   int ret;
 
   switch (rmd->artwork_fmt)
@@ -2023,7 +2009,10 @@ raop_metadata_send_artwork(struct raop_session *rs, struct evbuffer *evbuf, stru
 	return -1;
     }
 
-  ret = evbuffer_add(evbuf, EVBUFFER_DATA(rmd->artwork), EVBUFFER_LENGTH(rmd->artwork));
+  buf = evbuffer_pullup(rmd->artwork, -1);
+  len = evbuffer_get_length(rmd->artwork);
+
+  ret = evbuffer_add(evbuf, buf, len);
   if (ret != 0)
     {
       DPRINTF(E_LOG, L_RAOP, "Could not copy artwork for sending\n");
@@ -2041,9 +2030,14 @@ raop_metadata_send_artwork(struct raop_session *rs, struct evbuffer *evbuf, stru
 static int
 raop_metadata_send_metadata(struct raop_session *rs, struct evbuffer *evbuf, struct raop_metadata *rmd, char *rtptime)
 {
+  uint8_t *buf;
+  size_t len;
   int ret;
 
-  ret = evbuffer_add(evbuf, EVBUFFER_DATA(rmd->metadata), EVBUFFER_LENGTH(rmd->metadata));
+  buf = evbuffer_pullup(rmd->metadata, -1);
+  len = evbuffer_get_length(rmd->metadata);
+
+  ret = evbuffer_add(evbuf, buf, len);
   if (ret != 0)
     {
       DPRINTF(E_LOG, L_RAOP, "Could not copy metadata for sending\n");
@@ -2163,16 +2157,23 @@ raop_metadata_startup_send(struct raop_session *rs)
 }
 
 void
-raop_metadata_send(int id, uint64_t rtptime, uint64_t offset, int startup)
+raop_metadata_send(struct raop_metadata *rmd, uint64_t rtptime, uint64_t offset, int startup)
 {
   struct raop_session *rs;
-  struct raop_metadata *rmd;
   uint32_t delay;
   int ret;
 
-  rmd = raop_metadata_prepare(id, rtptime);
-  if (!rmd)
-    return;
+  rmd->start += rtptime;
+  rmd->end += rtptime;
+
+  /* Add the rmd to the metadata list */
+  if (metadata_tail)
+    metadata_tail->next = rmd;
+  else
+    {
+      metadata_head = rmd;
+      metadata_tail = rmd;
+    }
 
   for (rs = sessions; rs; rs = rs->next)
     {
@@ -2188,13 +2189,11 @@ raop_metadata_send(int id, uint64_t rtptime, uint64_t offset, int startup)
       if (ret < 0)
 	{
 	  raop_session_failure(rs);
-
 	  continue;
 	}
     }
 }
 
-
 /* Volume handling */
 static float
 raop_volume_convert(int volume, char *name)
@@ -2220,12 +2219,12 @@ raop_volume_convert(int volume, char *name)
 
   /* RAOP volume
    *  -144.0 is off
-   *  0 - -30.0 maps to 100 - 0
+   *  0 - 100 maps to -30.0 - 0
    */
-  if (volume == 0)
-    raop_volume = -144.0;
-  else
+  if (volume > 0 && volume <= 100)
     raop_volume = -30.0 + ((float)max_volume * (float)volume * 30.0) / (100.0 * RAOP_CONFIG_MAX_VOLUME);
+  else
+    raop_volume = -144.0;
 
   return raop_volume;
 }
@@ -2413,11 +2412,9 @@ raop_flush(raop_status_cb cb, uint64_t rtptime)
 
   if (pending > 0)
     {
-      evtimer_set(&flush_timer, raop_flush_timer_cb, NULL);
-      event_base_set(evbase_player, &flush_timer);
       evutil_timerclear(&tv);
       tv.tv_sec = 10;
-      evtimer_add(&flush_timer, &tv);
+      evtimer_add(flush_timer, &tv);
     }
 
   return pending;
@@ -2566,7 +2563,7 @@ raop_v2_timing_cb(int fd, short what, void *arg)
     }
 
  readd:
-  ret = event_add(&svc->ev, NULL);
+  ret = event_add(svc->ev, NULL);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_RAOP, "Couldn't re-add event for timing requests\n");
@@ -2655,16 +2652,16 @@ raop_v2_timing_start_one(struct raop_service *svc, int family)
 	break;
     }
 
-  event_set(&svc->ev, svc->fd, EV_READ, raop_v2_timing_cb, svc);
-  event_base_set(evbase_player, &svc->ev);
-  ret = event_add(&svc->ev, NULL);
-  if (ret < 0)
+  svc->ev = event_new(evbase_player, svc->fd, EV_READ, raop_v2_timing_cb, svc);
+  if (!svc->ev)
     {
-      DPRINTF(E_LOG, L_RAOP, "Couldn't add event for timing requests\n");
+      DPRINTF(E_LOG, L_RAOP, "Out of memory for raop_service event\n");
 
       goto out_fail;
     }
 
+  event_add(svc->ev, NULL);
+
   return 0;
 
  out_fail:
@@ -2678,11 +2675,11 @@ raop_v2_timing_start_one(struct raop_service *svc, int family)
 static void
 raop_v2_timing_stop(void)
 {
-  if (event_initialized(&timing_4svc.ev))
-    event_del(&timing_4svc.ev);
+  if (timing_4svc.ev)
+    event_free(timing_4svc.ev);
 
-  if (event_initialized(&timing_6svc.ev))
-    event_del(&timing_6svc.ev);
+  if (timing_6svc.ev)
+    event_free(timing_6svc.ev);
 
   close(timing_4svc.fd);
 
@@ -2898,7 +2895,7 @@ raop_v2_control_cb(int fd, short what, void *arg)
   raop_v2_resend_range(rs, seq_start, seq_len);
 
  readd:
-  ret = event_add(&svc->ev, NULL);
+  ret = event_add(svc->ev, NULL);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_RAOP, "Couldn't re-add event for control requests\n");
@@ -2987,16 +2984,16 @@ raop_v2_control_start_one(struct raop_service *svc, int family)
 	break;
     }
 
-  event_set(&svc->ev, svc->fd, EV_READ, raop_v2_control_cb, svc);
-  event_base_set(evbase_player, &svc->ev);
-  ret = event_add(&svc->ev, NULL);
-  if (ret < 0)
+  svc->ev = event_new(evbase_player, svc->fd, EV_READ, raop_v2_control_cb, svc);
+  if (!svc->ev)
     {
-      DPRINTF(E_LOG, L_RAOP, "Couldn't add event for control requests\n");
+      DPRINTF(E_LOG, L_RAOP, "Out of memory for control event\n");
 
       goto out_fail;
     }
 
+  event_add(svc->ev, NULL);
+
   return 0;
 
  out_fail:
@@ -3010,11 +3007,11 @@ raop_v2_control_start_one(struct raop_service *svc, int family)
 static void
 raop_v2_control_stop(void)
 {
-  if (event_initialized(&control_4svc.ev))
-    event_del(&control_4svc.ev);
+  if (control_4svc.ev)
+    event_free(control_4svc.ev);
 
-  if (event_initialized(&control_6svc.ev))
-    event_del(&control_6svc.ev);
+  if (control_6svc.ev)
+    event_free(control_6svc.ev);
 
   close(control_4svc.fd);
 
@@ -3921,8 +3918,7 @@ raop_playback_start(uint64_t next_pkt, struct timespec *ts)
 {
   struct raop_session *rs;
 
-  if (event_initialized(&flush_timer))
-    event_del(&flush_timer);
+  event_del(flush_timer);
 
   sync_counter = 0;
 
@@ -4044,12 +4040,20 @@ raop_init(int *v6enabled)
   if (ptr)
     *ptr = '\0';
 
+  flush_timer = evtimer_new(evbase_player, raop_flush_timer_cb, NULL);
+  if (!flush_timer)
+    {
+      DPRINTF(E_LOG, L_RAOP, "AirTunes v2 playback synchronization failed to start\n");
+
+      goto out_free_b64_iv;
+    }
+
   ret = raop_v2_timing_start(*v6enabled);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_RAOP, "AirTunes v2 time synchronization failed to start\n");
 
-      goto out_free_b64_iv;
+      goto out_free_flush_timer;
     }
 
   ret = raop_v2_control_start(*v6enabled);
@@ -4067,6 +4071,8 @@ raop_init(int *v6enabled)
 
  out_stop_timing:
   raop_v2_timing_stop();
+ out_free_flush_timer:
+  event_free(flush_timer);
  out_free_b64_iv:
   free(raop_aes_iv_b64);
  out_free_b64_key:
@@ -4089,8 +4095,10 @@ raop_deinit(void)
       raop_session_free(rs);
     }
 
-  raop_v2_timing_stop();
   raop_v2_control_stop();
+  raop_v2_timing_stop();
+
+  event_free(flush_timer);
 
   gcry_cipher_close(raop_aes_ctx);
 
