@@ -140,6 +140,8 @@ static void *g_libhandle;
 static enum spotify_state g_state;
 /* (not used) Tells which commmand is currently being processed */
 static struct spotify_command *g_cmd;
+// The global base playlist id (parent of all Spotify playlists in the db)
+static int g_base_plid;
 
 // Audio fifo
 static audio_fifo_t *g_audio_fifo;
@@ -184,6 +186,7 @@ typedef sp_error     (*fptr_sp_session_login_t)(sp_session *session, const char
 typedef sp_error     (*fptr_sp_session_relogin_t)(sp_session *session);
 typedef sp_error     (*fptr_sp_session_logout_t)(sp_session *session);
 typedef sp_error     (*fptr_sp_session_process_events_t)(sp_session *session, int *next_timeout);
+typedef sp_playlist* (*fptr_sp_session_starred_create_t)(sp_session *session);
 typedef sp_playlistcontainer* (*fptr_sp_session_playlistcontainer_t)(sp_session *session);
 typedef sp_error     (*fptr_sp_session_player_load_t)(sp_session *session, sp_track *track);
 typedef sp_error     (*fptr_sp_session_player_unload_t)(sp_session *session);
@@ -202,6 +205,7 @@ typedef sp_error     (*fptr_sp_playlist_remove_callbacks_t)(sp_playlist *playlis
 typedef int          (*fptr_sp_playlist_num_tracks_t)(sp_playlist *playlist);
 typedef sp_track*    (*fptr_sp_playlist_track_t)(sp_playlist *playlist, int index);
 typedef bool         (*fptr_sp_playlist_is_loaded_t)(sp_playlist *playlist);
+typedef int          (*fptr_sp_playlist_track_create_time_t)(sp_playlist *playlist, int index);
 
 typedef sp_error     (*fptr_sp_track_error_t)(sp_track *track);
 typedef bool         (*fptr_sp_track_is_loaded_t)(sp_track *track);
@@ -211,6 +215,7 @@ typedef int          (*fptr_sp_track_index_t)(sp_track *track);
 typedef int          (*fptr_sp_track_disc_t)(sp_track *track);
 typedef sp_album*    (*fptr_sp_track_album_t)(sp_track *track);
 typedef sp_track_availability (*fptr_sp_track_get_availability_t)(sp_session *session, sp_track *track);
+typedef bool         (*fptr_sp_track_is_starred_t)(sp_session *session, sp_track *track);
 
 typedef sp_link*     (*fptr_sp_link_create_from_playlist_t)(sp_playlist *playlist);
 typedef sp_link*     (*fptr_sp_link_create_from_track_t)(sp_track *track, int offset);
@@ -242,6 +247,7 @@ fptr_sp_session_release_t fptr_sp_session_release;
 fptr_sp_session_login_t fptr_sp_session_login;
 fptr_sp_session_relogin_t fptr_sp_session_relogin;
 fptr_sp_session_logout_t fptr_sp_session_logout;
+fptr_sp_session_starred_create_t fptr_sp_session_starred_create;
 fptr_sp_session_playlistcontainer_t fptr_sp_session_playlistcontainer;
 fptr_sp_session_process_events_t fptr_sp_session_process_events;
 fptr_sp_session_player_load_t fptr_sp_session_player_load;
@@ -261,6 +267,7 @@ fptr_sp_playlist_remove_callbacks_t fptr_sp_playlist_remove_callbacks;
 fptr_sp_playlist_num_tracks_t fptr_sp_playlist_num_tracks;
 fptr_sp_playlist_track_t fptr_sp_playlist_track;
 fptr_sp_playlist_is_loaded_t fptr_sp_playlist_is_loaded;
+fptr_sp_playlist_track_create_time_t fptr_sp_playlist_track_create_time;
 
 fptr_sp_track_error_t fptr_sp_track_error;
 fptr_sp_track_is_loaded_t fptr_sp_track_is_loaded;
@@ -270,6 +277,7 @@ fptr_sp_track_index_t fptr_sp_track_index;
 fptr_sp_track_disc_t fptr_sp_track_disc;
 fptr_sp_track_album_t fptr_sp_track_album;
 fptr_sp_track_get_availability_t fptr_sp_track_get_availability;
+fptr_sp_track_is_starred_t fptr_sp_track_is_starred;
 
 fptr_sp_link_create_from_playlist_t fptr_sp_link_create_from_playlist;
 fptr_sp_link_create_from_track_t fptr_sp_link_create_from_track;
@@ -320,6 +328,7 @@ fptr_assign_all()
    && (fptr_sp_session_preferred_bitrate = dlsym(h, "sp_session_preferred_bitrate"))
    && (fptr_sp_playlistcontainer_add_callbacks = dlsym(h, "sp_playlistcontainer_add_callbacks"))
    && (fptr_sp_playlistcontainer_num_playlists = dlsym(h, "sp_playlistcontainer_num_playlists"))
+   && (fptr_sp_session_starred_create = dlsym(h, "sp_session_starred_create"))
    && (fptr_sp_playlistcontainer_playlist = dlsym(h, "sp_playlistcontainer_playlist"))
    && (fptr_sp_playlist_add_callbacks = dlsym(h, "sp_playlist_add_callbacks"))
    && (fptr_sp_playlist_name = dlsym(h, "sp_playlist_name"))
@@ -327,6 +336,7 @@ fptr_assign_all()
    && (fptr_sp_playlist_num_tracks = dlsym(h, "sp_playlist_num_tracks"))
    && (fptr_sp_playlist_track = dlsym(h, "sp_playlist_track"))
    && (fptr_sp_playlist_is_loaded = dlsym(h, "sp_playlist_is_loaded"))
+   && (fptr_sp_playlist_track_create_time = dlsym(h, "sp_playlist_track_create_time"))
    && (fptr_sp_track_error = dlsym(h, "sp_track_error"))
    && (fptr_sp_track_is_loaded = dlsym(h, "sp_track_is_loaded"))
    && (fptr_sp_track_name = dlsym(h, "sp_track_name"))
@@ -335,6 +345,7 @@ fptr_assign_all()
    && (fptr_sp_track_disc = dlsym(h, "sp_track_disc"))
    && (fptr_sp_track_album = dlsym(h, "sp_track_album"))
    && (fptr_sp_track_get_availability = dlsym(h, "sp_track_get_availability"))
+   && (fptr_sp_track_is_starred = dlsym(h, "sp_track_is_starred"))
    && (fptr_sp_link_create_from_playlist = dlsym(h, "sp_link_create_from_playlist"))
    && (fptr_sp_link_create_from_track = dlsym(h, "sp_link_create_from_track"))
    && (fptr_sp_link_create_from_string = dlsym(h, "sp_link_create_from_string"))
@@ -459,11 +470,25 @@ thread_exit(void)
 /*            Should only be called from within the spotify thread           */
 
 static int
-spotify_metadata_get(sp_track *track, struct media_file_info *mfi)
+spotify_metadata_get(sp_track *track, struct media_file_info *mfi, const char *pltitle, int time_added)
 {
+  cfg_t *spotify_cfg;
+  bool artist_override;
+  bool starred_artist_override;
+  bool album_override;
+  bool starred_album_override;
   sp_album *album;
   sp_artist *artist;
   sp_albumtype albumtype;
+  bool starred;
+  int compilation;
+  char *albumname;
+
+  spotify_cfg = cfg_getsec(cfg, "spotify");
+  artist_override = cfg_getbool(spotify_cfg, "artist_override");
+  starred_artist_override = cfg_getbool(spotify_cfg, "starred_artist_override");
+  album_override = cfg_getbool(spotify_cfg, "album_override");
+  starred_album_override = cfg_getbool(spotify_cfg, "starred_album_override");
 
   album = fptr_sp_track_album(track);
   if (!album)
@@ -474,25 +499,61 @@ spotify_metadata_get(sp_track *track, struct media_file_info *mfi)
     return -1;
 
   albumtype = fptr_sp_album_type(album);
+  starred = fptr_sp_track_is_starred(g_sess, track);
+
+  /*
+   * Treat album as compilation if one of the following conditions is true:
+   * - spotfy album type is compilation
+   * - artist_override in config is set to true and track is not part of the starred playlist
+   * - starred_artist_override in config is set to true and track is part of the starred playlist
+   */
+  compilation = ((albumtype == SP_ALBUMTYPE_COMPILATION)
+		  || (starred && starred_artist_override)
+		  || (!starred && artist_override));
+
+  if ((starred && starred_album_override)
+      || (!starred && album_override))
+    albumname = strdup(pltitle);
+  else
+    albumname = strdup(fptr_sp_album_name(album));
 
   mfi->title       = strdup(fptr_sp_track_name(track));
-  mfi->album       = strdup(fptr_sp_album_name(album));
+  mfi->album       = albumname;
   mfi->artist      = strdup(fptr_sp_artist_name(artist));
   mfi->year        = fptr_sp_album_year(album);
   mfi->song_length = fptr_sp_track_duration(track);
   mfi->track       = fptr_sp_track_index(track);
   mfi->disc        = fptr_sp_track_disc(track);
-  mfi->compilation = (albumtype == SP_ALBUMTYPE_COMPILATION);
+  mfi->compilation = compilation;
   mfi->artwork     = ARTWORK_SPOTIFY;
   mfi->type        = strdup("spotify");
   mfi->codectype   = strdup("wav");
   mfi->description = strdup("Spotify audio");
+  mfi->time_added  = time_added;
+
+  DPRINTF(E_SPAM, L_SPOTIFY, "Metadata for track:\n"
+      "Title:       %s\n"
+      "Album:       %s\n"
+      "Artist:      %s\n"
+      "Year:        %u\n"
+      "Track:       %u\n"
+      "Disc:        %u\n"
+      "Compilation: %d\n"
+      "Starred:     %d\n",
+      mfi->title,
+      mfi->album,
+      mfi->artist,
+      mfi->year,
+      mfi->track,
+      mfi->disc,
+      mfi->compilation,
+      starred);
 
   return 0;
 }
 
 static int
-spotify_track_save(int plid, sp_track *track)
+spotify_track_save(int plid, sp_track *track, const char *pltitle, int time_added)
 {
   struct media_file_info mfi;
   sp_link *link;
@@ -501,8 +562,8 @@ spotify_track_save(int plid, sp_track *track)
 
   if (!fptr_sp_track_is_loaded(track))
     {
-      DPRINTF(E_INFO, L_SPOTIFY, "Metadata for track not ready yet\n");
-      return 0;
+      DPRINTF(E_LOG, L_SPOTIFY, "Track appears to no longer have the proper status\n");
+      return -1;
     }
 
   if (fptr_sp_track_get_availability(g_sess, track) != SP_TRACK_AVAILABILITY_AVAILABLE)
@@ -535,7 +596,7 @@ spotify_track_save(int plid, sp_track *track)
 
   memset(&mfi, 0, sizeof(struct media_file_info));
 
-  ret = spotify_metadata_get(track, &mfi);
+  ret = spotify_metadata_get(track, &mfi, pltitle, time_added);
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Metadata missing (but track should be loaded?): '%s'\n", fptr_sp_track_name(track));
@@ -558,21 +619,38 @@ spotify_playlist_save(sp_playlist *pl)
   sp_link *link;
   char url[1024];
   const char *name;
-  char title[512];
   int plid;
   int num_tracks;
+  char virtual_path[PATH_MAX];
+  int time;
   int ret;
   int i;
   
   if (!fptr_sp_playlist_is_loaded(pl))
     {
-      DPRINTF(E_DBG, L_SPOTIFY, "Playlist still not loaded - wait for rename callback\n");
+      DPRINTF(E_DBG, L_SPOTIFY, "Playlist still not loaded - will wait for next callback\n");
       return 0;
     }
 
   name = fptr_sp_playlist_name(pl);
+  num_tracks = fptr_sp_playlist_num_tracks(pl);
 
-  DPRINTF(E_INFO, L_SPOTIFY, "Saving playlist: '%s'\n", name);
+  // The starred playlist has an empty name, set it manually to "Starred"
+  if (*name == '\0')
+    name = "Starred";
+
+  for (i = 0; i < num_tracks; i++)
+    {
+      track = fptr_sp_playlist_track(pl, i);
+
+      if (track && !fptr_sp_track_is_loaded(track))
+	{
+	  DPRINTF(E_DBG, L_SPOTIFY, "All playlist tracks not loaded (will wait for next callback): %s\n", name);
+	  return 0;
+	}
+    }
+
+  DPRINTF(E_LOG, L_SPOTIFY, "Saving playlist (%d tracks): '%s'\n", num_tracks, name);
 
   /* Save playlist (playlists table) */
   link = fptr_sp_link_create_from_playlist(pl);
@@ -589,10 +667,9 @@ spotify_playlist_save(sp_playlist *pl)
     }
   fptr_sp_link_release(link);
 
-//  sleep(1); // Primitive way of preventing database locking (the mutex wasn't working)
-
   pli = db_pl_fetch_bypath(url);
-  snprintf(title, sizeof(title), "[s] %s", name);
+
+  snprintf(virtual_path, PATH_MAX, "/spotify:/%s", name);
 
   if (pli)
     {
@@ -600,32 +677,56 @@ spotify_playlist_save(sp_playlist *pl)
 
       plid = pli->id;
 
-      free_pli(pli, 0);
+      free(pli->title);
+      pli->title = strdup(name);
+      free(pli->virtual_path);
+      pli->virtual_path = strdup(virtual_path);
 
-      ret = db_pl_update(title, url, plid);
+      ret = db_pl_update(pli);
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error updating playlist ('%s', link %s)\n", name, url);
+
+	  free_pli(pli, 0);
 	  return -1;
 	}
 
-      db_pl_ping(plid);
       db_pl_clear_items(plid);
     }
   else
     {
       DPRINTF(E_DBG, L_SPOTIFY, "Adding playlist ('%s', link %s)\n", name, url);
 
-      ret = db_pl_add(title, url, &plid);
+      pli = (struct playlist_info *)malloc(sizeof(struct playlist_info));
+      if (!pli)
+	{
+	  DPRINTF(E_LOG, L_SCAN, "Out of memory\n");
+
+	  return -1;
+	}
+
+      memset(pli, 0, sizeof(struct playlist_info));
+
+      pli->type = PL_PLAIN;
+      pli->title = strdup(name);
+      pli->path = strdup(url);
+      pli->virtual_path = strdup(virtual_path);
+      pli->parent_id = g_base_plid;
+
+      ret = db_pl_add(pli, &plid);
       if ((ret < 0) || (plid < 1))
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error adding playlist ('%s', link %s, ret %d, plid %d)\n", name, url, ret, plid);
+
+	  free_pli(pli, 0);
 	  return -1;
 	}
     }
 
+  free_pli(pli, 0);
+
   /* Save tracks and playlistitems (files and playlistitems table) */
-  num_tracks = fptr_sp_playlist_num_tracks(pl);
+  db_transaction_begin();
   for (i = 0; i < num_tracks; i++)
     {
       track = fptr_sp_playlist_track(pl, i);
@@ -635,13 +736,16 @@ spotify_playlist_save(sp_playlist *pl)
 	  continue;
 	}
 
-      ret = spotify_track_save(plid, track);
+      time = fptr_sp_playlist_track_create_time(pl, i);
+
+      ret = spotify_track_save(plid, track, name, time);
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SPOTIFY, "Error saving track %d to playlist '%s' (id %d)\n", i, name, plid);
 	  continue;
 	}
     }
+  db_transaction_end();
 
   return plid;
 }
@@ -808,12 +912,12 @@ audio_fifo_flush(void)
 }
 
 static int
-playback_play(struct spotify_command *cmd)
+playback_setup(struct spotify_command *cmd)
 {
   sp_track *track;
   sp_error err;
 
-  DPRINTF(E_DBG, L_SPOTIFY, "Starting playback\n");
+  DPRINTF(E_DBG, L_SPOTIFY, "Setting up for playback\n");
 
   if (SP_CONNECTION_STATE_LOGGED_IN != fptr_sp_session_connectionstate(g_sess))
     {
@@ -843,6 +947,16 @@ playback_play(struct spotify_command *cmd)
 
   audio_fifo_flush();
 
+  return 0;
+}
+
+static int
+playback_play(struct spotify_command *cmd)
+{
+  sp_error err;
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Starting playback\n");
+
   err = fptr_sp_session_player_play(g_sess, 1);
   if (SP_ERROR_OK != err)
     {
@@ -876,25 +990,6 @@ playback_pause(struct spotify_command *cmd)
   return 0;
 }
 
-static int
-playback_resume(struct spotify_command *cmd)
-{
-  sp_error err;
-
-  DPRINTF(E_DBG, L_SPOTIFY, "Resuming playback\n");
-
-  err = fptr_sp_session_player_play(g_sess, 1);
-  if (SP_ERROR_OK != err)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Playback resume failed: %s\n", fptr_sp_error_message(err));
-      return -1;
-    }
-
-  g_state = SPOTIFY_STATE_PLAYING;
-
-  return 0;
-}
-
 static int
 playback_stop(struct spotify_command *cmd)
 {
@@ -967,7 +1062,7 @@ audio_get(struct spotify_command *cmd)
 
   // If spotify was paused begin by resuming playback
   if (g_state == SPOTIFY_STATE_PAUSED)
-    playback_resume(NULL);
+    playback_play(NULL);
 
   pthread_mutex_lock(&g_audio_fifo->mutex);
 
@@ -1162,8 +1257,11 @@ artwork_get(struct spotify_command *cmd)
 static void
 logged_in(sp_session *sess, sp_error error)
 {
+  cfg_t *spotify_cfg;
   sp_playlist *pl;
   sp_playlistcontainer *pc;
+  struct playlist_info pli;
+  int ret;
   int i;
 
   if (SP_ERROR_OK != error)
@@ -1176,6 +1274,27 @@ logged_in(sp_session *sess, sp_error error)
 
   db_spotify_purge();
 
+  pl = fptr_sp_session_starred_create(sess);
+  fptr_sp_playlist_add_callbacks(pl, &pl_callbacks, NULL);
+
+  spotify_cfg = cfg_getsec(cfg, "spotify");
+  if (! cfg_getbool(spotify_cfg, "base_playlist_disable"))
+    {
+      memset(&pli, 0, sizeof(struct playlist_info));
+      pli.title = "Spotify";
+      pli.type = PL_FOLDER;
+      pli.path = "spotify:playlistfolder";
+
+      ret = db_pl_add(&pli, &g_base_plid);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_SPOTIFY, "Error adding base playlist\n");
+	  return;
+	}
+    }
+  else
+    g_base_plid = 0;
+
   pc = fptr_sp_session_playlistcontainer(sess);
 
   fptr_sp_playlistcontainer_add_callbacks(pc, &pc_callbacks, NULL);
@@ -1498,13 +1617,13 @@ notify_cb(int fd, short what, void *arg)
 
 /* Thread: player */
 int
-spotify_playback_play(struct media_file_info *mfi)
+spotify_playback_setup(struct media_file_info *mfi)
 {
   struct spotify_command cmd;
   sp_link *link;
   int ret;
 
-  DPRINTF(E_DBG, L_SPOTIFY, "Playback request\n");
+  DPRINTF(E_DBG, L_SPOTIFY, "Playback setup request\n");
 
   link = fptr_sp_link_create_from_string(mfi->path);
   if (!link)
@@ -1515,7 +1634,7 @@ spotify_playback_play(struct media_file_info *mfi)
 
   command_init(&cmd);
 
-  cmd.func = playback_play;
+  cmd.func = playback_setup;
   cmd.arg.link = link;
 
   ret = sync_command(&cmd);
@@ -1525,6 +1644,46 @@ spotify_playback_play(struct media_file_info *mfi)
   return ret;
 }
 
+int
+spotify_playback_play()
+{
+  struct spotify_command cmd;
+  int ret;
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Playback request\n");
+
+  command_init(&cmd);
+
+  cmd.func = playback_play;
+  cmd.arg.noarg = NULL;
+
+  ret = sync_command(&cmd);
+
+  command_deinit(&cmd);
+
+  return ret;
+}
+
+int
+spotify_playback_pause()
+{
+  struct spotify_command cmd;
+  int ret;
+
+  DPRINTF(E_DBG, L_SPOTIFY, "Pause request\n");
+
+  command_init(&cmd);
+
+  cmd.func = playback_pause;
+  cmd.arg.noarg = NULL;
+
+  ret = sync_command(&cmd);
+
+  command_deinit(&cmd);
+
+  return ret;
+}
+
 /* Thread: libspotify */
 void
 spotify_playback_pause_nonblock(void)
@@ -1550,27 +1709,6 @@ spotify_playback_pause_nonblock(void)
   nonblock_command(cmd);
 }
 
-/* Not used */
-int
-spotify_playback_resume(void)
-{
-  struct spotify_command cmd;
-  int ret;
-
-  DPRINTF(E_DBG, L_SPOTIFY, "Resume request\n");
-
-  command_init(&cmd);
-
-  cmd.func = playback_resume;
-  cmd.arg.noarg = NULL;
-
-  ret = sync_command(&cmd);
-
-  command_deinit(&cmd);
-
-  return ret;
-}
-
 /* Thread: player and libspotify */
 int
 spotify_playback_stop(void)
@@ -1866,33 +2004,21 @@ spotify_init(void)
   if (ret < 0)
     goto assign_fail;
 
-# if defined(__linux__)
   ret = pipe2(g_exit_pipe, O_CLOEXEC);
-# else
-  ret = pipe(g_exit_pipe);
-# endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Could not create pipe: %s\n", strerror(errno));
       goto exit_fail;
     }
 
-# if defined(__linux__)
   ret = pipe2(g_cmd_pipe, O_CLOEXEC);
-# else
-  ret = pipe(g_cmd_pipe);
-# endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Could not create command pipe: %s\n", strerror(errno));
       goto cmd_fail;
     }
 
-# if defined(__linux__)
   ret = pipe2(g_notify_pipe, O_CLOEXEC);
-# else
-  ret = pipe(g_notify_pipe);
-# endif
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_SPOTIFY, "Could not notify command pipe: %s\n", strerror(errno));
@@ -1906,7 +2032,6 @@ spotify_init(void)
       goto evbase_fail;
     }
 
-#ifdef HAVE_LIBEVENT2
   g_exitev = event_new(evbase_spotify, g_exit_pipe[0], EV_READ, exit_cb, NULL);
   if (!g_exitev)
     {
@@ -1927,34 +2052,6 @@ spotify_init(void)
       DPRINTF(E_LOG, L_SPOTIFY, "Could not create notify event\n");
       goto evnew_fail;
     }
-#else
-  g_exitev = (struct event *)malloc(sizeof(struct event));
-  if (!g_exitev)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not create exit event\n");
-      goto evnew_fail;
-    }
-  event_set(g_exitev, g_exit_pipe[0], EV_READ, exit_cb, NULL);
-  event_base_set(evbase_spotify, g_exitev);
-
-  g_cmdev = (struct event *)malloc(sizeof(struct event));
-  if (!g_cmdev)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not create cmd event\n");
-      goto evnew_fail;
-    }
-  event_set(g_cmdev, g_cmd_pipe[0], EV_READ, command_cb, NULL);
-  event_base_set(evbase_spotify, g_cmdev);
-
-  g_notifyev = (struct event *)malloc(sizeof(struct event));
-  if (!g_notifyev)
-    {
-      DPRINTF(E_LOG, L_SPOTIFY, "Could not create notify event\n");
-      goto evnew_fail;
-    }
-  event_set(g_notifyev, g_notify_pipe[0], EV_READ | EV_TIMEOUT, notify_cb, NULL);
-  event_base_set(evbase_spotify, g_notifyev);
-#endif
 
   event_add(g_exitev, NULL);
   event_add(g_cmdev, NULL);
